/**
 * react-dyn-tabs - React dynamic tabs with full API
 *
 * @version v6.2.2
 * @homepage https://dev-javascript.github.io/react-dyn-tabs/
 * @author dev-javascript javascript.code.dev@gmail.com
 * @license MIT
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('react'), require('prop-types')) :
  typeof define === 'function' && define.amd ? define(['react', 'prop-types'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.MoreButtonPlugin = factory(global.React));
})(this, (function (React) { 'use strict';

  var Api$2 = function Api(_ref) {
    var getResizerIns = _ref.getResizerIns,
      btnRef = _ref.btnRef,
      ctx = _ref.ctx,
      setHiddenTabIDs = _ref.setHiddenTabIDs;
    this.api = ctx;
    this.tablistEl = null;
    this.getResizerIns = getResizerIns;
    this.tablistContainerEl = null;
    this.tabs = null;
    this.tabsCount = null;
    this.btnRef = btnRef;
    this._setHiddenTabIDs = setHiddenTabIDs;
  };
  Object.assign(Api$2.prototype, {
    setResizer: function setResizer() {
      this.tablistEl = this.btnRef.current.previousSibling;
      this.tablistContainerEl = this.tablistEl.parentElement.parentElement;
      this.tablistViewEl = this.tablistContainerEl.parentElement;
      this._resizer = this.getResizerIns({
        containerElement: this.tablistContainerEl,
        buttonElement: this.btnRef.current,
        tablistElement: this.tablistEl
      });
    },
    installResizer: function installResizer(resizeDetectorIns) {
      var _this = this;
      resizeDetectorIns.debncListenTo(this.tablistViewEl, function () {
        return _this.resize();
      });
    },
    uninstallResizer: function uninstallResizer(resizeDetectorIns) {
      if (this.tablistViewEl && resizeDetectorIns) resizeDetectorIns.uninstall(this.tablistViewEl);
    },
    _validateTabsCount: function _validateTabsCount(data) {
      this.tabs = this.tablistEl.children;
      this.tabsCount = this.tabs.length;
      if (data.openTabIDs.length !== this.tabsCount) {
        throw new Error("tablist children's length is not equal to openTabIDs length");
      }
      return true;
    },
    resize: function resize() {
      var ins = this.api;
      var data = ins.getData();
      if (this._validateTabsCount(data) === false) {
        return this._setHiddenTabIDs('');
      }
      var openTabIDs = data.openTabIDs,
        selectedTabID = data.selectedTabID;
      var selectedTabIndex = openTabIDs.indexOf(selectedTabID);
      var hiddenTabs = this._resizer.resize(selectedTabIndex, ins.getOption('direction'), ins.getOption('isVertical'));
      if (!hiddenTabs.length) {
        return this._setHiddenTabIDs('');
      }
      this._setHiddenTabIDs(hiddenTabs.map(function (_ref2) {
        var index = _ref2.index;
        return openTabIDs[index];
      }).toString());
    }
  });

  function getDefaultExportFromCjs (x) {
  	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
  }

  var collectionUtils = {exports: {}};

  var utils$3 = collectionUtils.exports = {};

  /**
   * Loops through the collection and calls the callback for each element. if the callback returns truthy, the loop is broken and returns the same value.
   * @public
   * @param {*} collection The collection to loop through. Needs to have a length property set and have indices set from 0 to length - 1.
   * @param {function} callback The callback to be called for each element. The element will be given as a parameter to the callback. If this callback returns truthy, the loop is broken and the same value is returned.
   * @returns {*} The value that a callback has returned (if truthy). Otherwise nothing.
   */
  utils$3.forEach = function(collection, callback) {
      for(var i = 0; i < collection.length; i++) {
          var result = callback(collection[i]);
          if(result) {
              return result;
          }
      }
  };

  var collectionUtilsExports = collectionUtils.exports;

  var elementUtils = function(options) {
      var getState = options.stateHandler.getState;

      /**
       * Tells if the element has been made detectable and ready to be listened for resize events.
       * @public
       * @param {element} The element to check.
       * @returns {boolean} True or false depending on if the element is detectable or not.
       */
      function isDetectable(element) {
          var state = getState(element);
          return state && !!state.isDetectable;
      }

      /**
       * Marks the element that it has been made detectable and ready to be listened for resize events.
       * @public
       * @param {element} The element to mark.
       */
      function markAsDetectable(element) {
          getState(element).isDetectable = true;
      }

      /**
       * Tells if the element is busy or not.
       * @public
       * @param {element} The element to check.
       * @returns {boolean} True or false depending on if the element is busy or not.
       */
      function isBusy(element) {
          return !!getState(element).busy;
      }

      /**
       * Marks the object is busy and should not be made detectable.
       * @public
       * @param {element} element The element to mark.
       * @param {boolean} busy If the element is busy or not.
       */
      function markBusy(element, busy) {
          getState(element).busy = !!busy;
      }

      return {
          isDetectable: isDetectable,
          markAsDetectable: markAsDetectable,
          isBusy: isBusy,
          markBusy: markBusy
      };
  };

  var listenerHandler = function(idHandler) {
      var eventListeners = {};

      /**
       * Gets all listeners for the given element.
       * @public
       * @param {element} element The element to get all listeners for.
       * @returns All listeners for the given element.
       */
      function getListeners(element) {
          var id = idHandler.get(element);

          if (id === undefined) {
              return [];
          }

          return eventListeners[id] || [];
      }

      /**
       * Stores the given listener for the given element. Will not actually add the listener to the element.
       * @public
       * @param {element} element The element that should have the listener added.
       * @param {function} listener The callback that the element has added.
       */
      function addListener(element, listener) {
          var id = idHandler.get(element);

          if(!eventListeners[id]) {
              eventListeners[id] = [];
          }

          eventListeners[id].push(listener);
      }

      function removeListener(element, listener) {
          var listeners = getListeners(element);
          for (var i = 0, len = listeners.length; i < len; ++i) {
              if (listeners[i] === listener) {
                listeners.splice(i, 1);
                break;
              }
          }
      }

      function removeAllListeners(element) {
        var listeners = getListeners(element);
        if (!listeners) { return; }
        listeners.length = 0;
      }

      return {
          get: getListeners,
          add: addListener,
          removeListener: removeListener,
          removeAllListeners: removeAllListeners
      };
  };

  var idGenerator = function() {
      var idCount = 1;

      /**
       * Generates a new unique id in the context.
       * @public
       * @returns {number} A unique id in the context.
       */
      function generate() {
          return idCount++;
      }

      return {
          generate: generate
      };
  };

  var idHandler = function(options) {
      var idGenerator     = options.idGenerator;
      var getState        = options.stateHandler.getState;

      /**
       * Gets the resize detector id of the element.
       * @public
       * @param {element} element The target element to get the id of.
       * @returns {string|number|null} The id of the element. Null if it has no id.
       */
      function getId(element) {
          var state = getState(element);

          if (state && state.id !== undefined) {
              return state.id;
          }

          return null;
      }

      /**
       * Sets the resize detector id of the element. Requires the element to have a resize detector state initialized.
       * @public
       * @param {element} element The target element to set the id of.
       * @returns {string|number|null} The id of the element.
       */
      function setId(element) {
          var state = getState(element);

          if (!state) {
              throw new Error("setId required the element to have a resize detection state.");
          }

          var id = idGenerator.generate();

          state.id = id;

          return id;
      }

      return {
          get: getId,
          set: setId
      };
  };

  /* global console: false */

  /**
   * Reporter that handles the reporting of logs, warnings and errors.
   * @public
   * @param {boolean} quiet Tells if the reporter should be quiet or not.
   */
  var reporter = function(quiet) {
      function noop() {
          //Does nothing.
      }

      var reporter = {
          log: noop,
          warn: noop,
          error: noop
      };

      if(!quiet && window.console) {
          var attachFunction = function(reporter, name) {
              //The proxy is needed to be able to call the method with the console context,
              //since we cannot use bind.
              reporter[name] = function reporterProxy() {
                  var f = console[name];
                  if (f.apply) { //IE9 does not support console.log.apply :)
                      f.apply(console, arguments);
                  } else {
                      for (var i = 0; i < arguments.length; i++) {
                          f(arguments[i]);
                      }
                  }
              };
          };

          attachFunction(reporter, "log");
          attachFunction(reporter, "warn");
          attachFunction(reporter, "error");
      }

      return reporter;
  };

  var browserDetector$2 = {exports: {}};

  var detector = browserDetector$2.exports = {};

  detector.isIE = function(version) {
      function isAnyIeVersion() {
          var agent = navigator.userAgent.toLowerCase();
          return agent.indexOf("msie") !== -1 || agent.indexOf("trident") !== -1 || agent.indexOf(" edge/") !== -1;
      }

      if(!isAnyIeVersion()) {
          return false;
      }

      if(!version) {
          return true;
      }

      //Shamelessly stolen from https://gist.github.com/padolsey/527683
      var ieVersion = (function(){
          var undef,
              v = 3,
              div = document.createElement("div"),
              all = div.getElementsByTagName("i");

          do {
              div.innerHTML = "<!--[if gt IE " + (++v) + "]><i></i><![endif]-->";
          }
          while (all[0]);

          return v > 4 ? v : undef;
      }());

      return version === ieVersion;
  };

  detector.isLegacyOpera = function() {
      return !!window.opera;
  };

  var browserDetectorExports = browserDetector$2.exports;

  var utils$2 = {exports: {}};

  var utils$1 = utils$2.exports = {};

  utils$1.getOption = getOption$1;

  function getOption$1(options, name, defaultValue) {
      var value = options[name];

      if((value === undefined || value === null) && defaultValue !== undefined) {
          return defaultValue;
      }

      return value;
  }

  var utilsExports = utils$2.exports;

  var utils = utilsExports;

  var batchProcessor = function batchProcessorMaker(options) {
      options             = options || {};
      var reporter        = options.reporter;
      var asyncProcess    = utils.getOption(options, "async", true);
      var autoProcess     = utils.getOption(options, "auto", true);

      if(autoProcess && !asyncProcess) {
          reporter && reporter.warn("Invalid options combination. auto=true and async=false is invalid. Setting async=true.");
          asyncProcess = true;
      }

      var batch = Batch();
      var asyncFrameHandler;
      var isProcessing = false;

      function addFunction(level, fn) {
          if(!isProcessing && autoProcess && asyncProcess && batch.size() === 0) {
              // Since this is async, it is guaranteed to be executed after that the fn is added to the batch.
              // This needs to be done before, since we're checking the size of the batch to be 0.
              processBatchAsync();
          }

          batch.add(level, fn);
      }

      function processBatch() {
          // Save the current batch, and create a new batch so that incoming functions are not added into the currently processing batch.
          // Continue processing until the top-level batch is empty (functions may be added to the new batch while processing, and so on).
          isProcessing = true;
          while (batch.size()) {
              var processingBatch = batch;
              batch = Batch();
              processingBatch.process();
          }
          isProcessing = false;
      }

      function forceProcessBatch(localAsyncProcess) {
          if (isProcessing) {
              return;
          }

          if(localAsyncProcess === undefined) {
              localAsyncProcess = asyncProcess;
          }

          if(asyncFrameHandler) {
              cancelFrame(asyncFrameHandler);
              asyncFrameHandler = null;
          }

          if(localAsyncProcess) {
              processBatchAsync();
          } else {
              processBatch();
          }
      }

      function processBatchAsync() {
          asyncFrameHandler = requestFrame(processBatch);
      }

      function cancelFrame(listener) {
          // var cancel = window.cancelAnimationFrame || window.mozCancelAnimationFrame || window.webkitCancelAnimationFrame || window.clearTimeout;
          var cancel = clearTimeout;
          return cancel(listener);
      }

      function requestFrame(callback) {
          // var raf = window.requestAnimationFrame || window.mozRequestAnimationFrame || window.webkitRequestAnimationFrame || function(fn) { return window.setTimeout(fn, 20); };
          var raf = function(fn) { return setTimeout(fn, 0); };
          return raf(callback);
      }

      return {
          add: addFunction,
          force: forceProcessBatch
      };
  };

  function Batch() {
      var batch       = {};
      var size        = 0;
      var topLevel    = 0;
      var bottomLevel = 0;

      function add(level, fn) {
          if(!fn) {
              fn = level;
              level = 0;
          }

          if(level > topLevel) {
              topLevel = level;
          } else if(level < bottomLevel) {
              bottomLevel = level;
          }

          if(!batch[level]) {
              batch[level] = [];
          }

          batch[level].push(fn);
          size++;
      }

      function process() {
          for(var level = bottomLevel; level <= topLevel; level++) {
              var fns = batch[level];

              for(var i = 0; i < fns.length; i++) {
                  var fn = fns[i];
                  fn();
              }
          }
      }

      function getSize() {
          return size;
      }

      return {
          add: add,
          process: process,
          size: getSize
      };
  }

  var prop = "_erd";

  function initState(element) {
      element[prop] = {};
      return getState(element);
  }

  function getState(element) {
      return element[prop];
  }

  function cleanState(element) {
      delete element[prop];
  }

  var stateHandler$1 = {
      initState: initState,
      getState: getState,
      cleanState: cleanState
  };

  /**
   * Resize detection strategy that injects objects to elements in order to detect resize events.
   * Heavily inspired by: http://www.backalleycoder.com/2013/03/18/cross-browser-event-based-element-resize-detection/
   */

  var browserDetector$1 = browserDetectorExports;

  var object = function(options) {
      options             = options || {};
      var reporter        = options.reporter;
      var batchProcessor  = options.batchProcessor;
      var getState        = options.stateHandler.getState;

      if(!reporter) {
          throw new Error("Missing required dependency: reporter.");
      }

      /**
       * Adds a resize event listener to the element.
       * @public
       * @param {element} element The element that should have the listener added.
       * @param {function} listener The listener callback to be called for each resize event of the element. The element will be given as a parameter to the listener callback.
       */
      function addListener(element, listener) {
          function listenerProxy() {
              listener(element);
          }

          if(browserDetector$1.isIE(8)) {
              //IE 8 does not support object, but supports the resize event directly on elements.
              getState(element).object = {
                  proxy: listenerProxy
              };
              element.attachEvent("onresize", listenerProxy);
          } else {
              var object = getObject(element);

              if(!object) {
                  throw new Error("Element is not detectable by this strategy.");
              }

              object.contentDocument.defaultView.addEventListener("resize", listenerProxy);
          }
      }

      function buildCssTextString(rules) {
          var seperator = options.important ? " !important; " : "; ";

          return (rules.join(seperator) + seperator).trim();
      }

      /**
       * Makes an element detectable and ready to be listened for resize events. Will call the callback when the element is ready to be listened for resize changes.
       * @private
       * @param {object} options Optional options object.
       * @param {element} element The element to make detectable
       * @param {function} callback The callback to be called when the element is ready to be listened for resize changes. Will be called with the element as first parameter.
       */
      function makeDetectable(options, element, callback) {
          if (!callback) {
              callback = element;
              element = options;
              options = null;
          }

          options = options || {};
          options.debug;

          function injectObject(element, callback) {
              var OBJECT_STYLE = buildCssTextString(["display: block", "position: absolute", "top: 0", "left: 0", "width: 100%", "height: 100%", "border: none", "padding: 0", "margin: 0", "opacity: 0", "z-index: -1000", "pointer-events: none"]);

              //The target element needs to be positioned (everything except static) so the absolute positioned object will be positioned relative to the target element.

              // Position altering may be performed directly or on object load, depending on if style resolution is possible directly or not.
              var positionCheckPerformed = false;

              // The element may not yet be attached to the DOM, and therefore the style object may be empty in some browsers.
              // Since the style object is a reference, it will be updated as soon as the element is attached to the DOM.
              var style = window.getComputedStyle(element);
              var width = element.offsetWidth;
              var height = element.offsetHeight;

              getState(element).startSize = {
                  width: width,
                  height: height
              };

              function mutateDom() {
                  function alterPositionStyles() {
                      if(style.position === "static") {
                          element.style.setProperty("position", "relative", options.important ? "important" : "");

                          var removeRelativeStyles = function(reporter, element, style, property) {
                              function getNumericalValue(value) {
                                  return value.replace(/[^-\d\.]/g, "");
                              }

                              var value = style[property];

                              if(value !== "auto" && getNumericalValue(value) !== "0") {
                                  reporter.warn("An element that is positioned static has style." + property + "=" + value + " which is ignored due to the static positioning. The element will need to be positioned relative, so the style." + property + " will be set to 0. Element: ", element);
                                  element.style.setProperty(property, "0", options.important ? "important" : "");
                              }
                          };

                          //Check so that there are no accidental styles that will make the element styled differently now that is is relative.
                          //If there are any, set them to 0 (this should be okay with the user since the style properties did nothing before [since the element was positioned static] anyway).
                          removeRelativeStyles(reporter, element, style, "top");
                          removeRelativeStyles(reporter, element, style, "right");
                          removeRelativeStyles(reporter, element, style, "bottom");
                          removeRelativeStyles(reporter, element, style, "left");
                      }
                  }

                  function onObjectLoad() {
                      // The object has been loaded, which means that the element now is guaranteed to be attached to the DOM.
                      if (!positionCheckPerformed) {
                          alterPositionStyles();
                      }

                      /*jshint validthis: true */

                      function getDocument(element, callback) {
                          //Opera 12 seem to call the object.onload before the actual document has been created.
                          //So if it is not present, poll it with an timeout until it is present.
                          //TODO: Could maybe be handled better with object.onreadystatechange or similar.
                          if(!element.contentDocument) {
                              var state = getState(element);
                              if (state.checkForObjectDocumentTimeoutId) {
                                  window.clearTimeout(state.checkForObjectDocumentTimeoutId);
                              }
                              state.checkForObjectDocumentTimeoutId = setTimeout(function checkForObjectDocument() {
                                  state.checkForObjectDocumentTimeoutId = 0;
                                  getDocument(element, callback);
                              }, 100);

                              return;
                          }

                          callback(element.contentDocument);
                      }

                      //Mutating the object element here seems to fire another load event.
                      //Mutating the inner document of the object element is fine though.
                      var objectElement = this;

                      //Create the style element to be added to the object.
                      getDocument(objectElement, function onObjectDocumentReady(objectDocument) {
                          //Notify that the element is ready to be listened to.
                          callback(element);
                      });
                  }

                  // The element may be detached from the DOM, and some browsers does not support style resolving of detached elements.
                  // The alterPositionStyles needs to be delayed until we know the element has been attached to the DOM (which we are sure of when the onObjectLoad has been fired), if style resolution is not possible.
                  if (style.position !== "") {
                      alterPositionStyles();
                      positionCheckPerformed = true;
                  }

                  //Add an object element as a child to the target element that will be listened to for resize events.
                  var object = document.createElement("object");
                  object.style.cssText = OBJECT_STYLE;
                  object.tabIndex = -1;
                  object.type = "text/html";
                  object.setAttribute("aria-hidden", "true");
                  object.onload = onObjectLoad;

                  //Safari: This must occur before adding the object to the DOM.
                  //IE: Does not like that this happens before, even if it is also added after.
                  if(!browserDetector$1.isIE()) {
                      object.data = "about:blank";
                  }

                  if (!getState(element)) {
                      // The element has been uninstalled before the actual loading happened.
                      return;
                  }

                  element.appendChild(object);
                  getState(element).object = object;

                  //IE: This must occur after adding the object to the DOM.
                  if(browserDetector$1.isIE()) {
                      object.data = "about:blank";
                  }
              }

              if(batchProcessor) {
                  batchProcessor.add(mutateDom);
              } else {
                  mutateDom();
              }
          }

          if(browserDetector$1.isIE(8)) {
              //IE 8 does not support objects properly. Luckily they do support the resize event.
              //So do not inject the object and notify that the element is already ready to be listened to.
              //The event handler for the resize event is attached in the utils.addListener instead.
              callback(element);
          } else {
              injectObject(element, callback);
          }
      }

      /**
       * Returns the child object of the target element.
       * @private
       * @param {element} element The target element.
       * @returns The object element of the target.
       */
      function getObject(element) {
          return getState(element).object;
      }

      function uninstall(element) {
          if (!getState(element)) {
              return;
          }

          var object = getObject(element);

          if (!object) {
              return;
          }

          if (browserDetector$1.isIE(8)) {
              element.detachEvent("onresize", object.proxy);
          } else {
              element.removeChild(object);
          }

          if (getState(element).checkForObjectDocumentTimeoutId) {
              window.clearTimeout(getState(element).checkForObjectDocumentTimeoutId);
          }

          delete getState(element).object;
      }

      return {
          makeDetectable: makeDetectable,
          addListener: addListener,
          uninstall: uninstall
      };
  };

  /**
   * Resize detection strategy that injects divs to elements in order to detect resize events on scroll events.
   * Heavily inspired by: https://github.com/marcj/css-element-queries/blob/master/src/ResizeSensor.js
   */

  var forEach$1 = collectionUtilsExports.forEach;

  var scroll = function(options) {
      options             = options || {};
      var reporter        = options.reporter;
      var batchProcessor  = options.batchProcessor;
      var getState        = options.stateHandler.getState;
      options.stateHandler.hasState;
      var idHandler       = options.idHandler;

      if (!batchProcessor) {
          throw new Error("Missing required dependency: batchProcessor");
      }

      if (!reporter) {
          throw new Error("Missing required dependency: reporter.");
      }

      //TODO: Could this perhaps be done at installation time?
      var scrollbarSizes = getScrollbarSizes();

      var styleId = "erd_scroll_detection_scrollbar_style";
      var detectionContainerClass = "erd_scroll_detection_container";

      function initDocument(targetDocument) {
          // Inject the scrollbar styling that prevents them from appearing sometimes in Chrome.
          // The injected container needs to have a class, so that it may be styled with CSS (pseudo elements).
          injectScrollStyle(targetDocument, styleId, detectionContainerClass);
      }

      initDocument(window.document);

      function buildCssTextString(rules) {
          var seperator = options.important ? " !important; " : "; ";

          return (rules.join(seperator) + seperator).trim();
      }

      function getScrollbarSizes() {
          var width = 500;
          var height = 500;

          var child = document.createElement("div");
          child.style.cssText = buildCssTextString(["position: absolute", "width: " + width*2 + "px", "height: " + height*2 + "px", "visibility: hidden", "margin: 0", "padding: 0"]);

          var container = document.createElement("div");
          container.style.cssText = buildCssTextString(["position: absolute", "width: " + width + "px", "height: " + height + "px", "overflow: scroll", "visibility: none", "top: " + -width*3 + "px", "left: " + -height*3 + "px", "visibility: hidden", "margin: 0", "padding: 0"]);

          container.appendChild(child);

          document.body.insertBefore(container, document.body.firstChild);

          var widthSize = width - container.clientWidth;
          var heightSize = height - container.clientHeight;

          document.body.removeChild(container);

          return {
              width: widthSize,
              height: heightSize
          };
      }

      function injectScrollStyle(targetDocument, styleId, containerClass) {
          function injectStyle(style, method) {
              method = method || function (element) {
                  targetDocument.head.appendChild(element);
              };

              var styleElement = targetDocument.createElement("style");
              styleElement.innerHTML = style;
              styleElement.id = styleId;
              method(styleElement);
              return styleElement;
          }

          if (!targetDocument.getElementById(styleId)) {
              var containerAnimationClass = containerClass + "_animation";
              var containerAnimationActiveClass = containerClass + "_animation_active";
              var style = "/* Created by the element-resize-detector library. */\n";
              style += "." + containerClass + " > div::-webkit-scrollbar { " + buildCssTextString(["display: none"]) + " }\n\n";
              style += "." + containerAnimationActiveClass + " { " + buildCssTextString(["-webkit-animation-duration: 0.1s", "animation-duration: 0.1s", "-webkit-animation-name: " + containerAnimationClass, "animation-name: " + containerAnimationClass]) + " }\n";
              style += "@-webkit-keyframes " + containerAnimationClass +  " { 0% { opacity: 1; } 50% { opacity: 0; } 100% { opacity: 1; } }\n";
              style += "@keyframes " + containerAnimationClass +          " { 0% { opacity: 1; } 50% { opacity: 0; } 100% { opacity: 1; } }";
              injectStyle(style);
          }
      }

      function addAnimationClass(element) {
          element.className += " " + detectionContainerClass + "_animation_active";
      }

      function addEvent(el, name, cb) {
          if (el.addEventListener) {
              el.addEventListener(name, cb);
          } else if(el.attachEvent) {
              el.attachEvent("on" + name, cb);
          } else {
              return reporter.error("[scroll] Don't know how to add event listeners.");
          }
      }

      function removeEvent(el, name, cb) {
          if (el.removeEventListener) {
              el.removeEventListener(name, cb);
          } else if(el.detachEvent) {
              el.detachEvent("on" + name, cb);
          } else {
              return reporter.error("[scroll] Don't know how to remove event listeners.");
          }
      }

      function getExpandElement(element) {
          return getState(element).container.childNodes[0].childNodes[0].childNodes[0];
      }

      function getShrinkElement(element) {
          return getState(element).container.childNodes[0].childNodes[0].childNodes[1];
      }

      /**
       * Adds a resize event listener to the element.
       * @public
       * @param {element} element The element that should have the listener added.
       * @param {function} listener The listener callback to be called for each resize event of the element. The element will be given as a parameter to the listener callback.
       */
      function addListener(element, listener) {
          var listeners = getState(element).listeners;

          if (!listeners.push) {
              throw new Error("Cannot add listener to an element that is not detectable.");
          }

          getState(element).listeners.push(listener);
      }

      /**
       * Makes an element detectable and ready to be listened for resize events. Will call the callback when the element is ready to be listened for resize changes.
       * @private
       * @param {object} options Optional options object.
       * @param {element} element The element to make detectable
       * @param {function} callback The callback to be called when the element is ready to be listened for resize changes. Will be called with the element as first parameter.
       */
      function makeDetectable(options, element, callback) {
          if (!callback) {
              callback = element;
              element = options;
              options = null;
          }

          options = options || {};

          function debug() {
              if (options.debug) {
                  var args = Array.prototype.slice.call(arguments);
                  args.unshift(idHandler.get(element), "Scroll: ");
                  if (reporter.log.apply) {
                      reporter.log.apply(null, args);
                  } else {
                      for (var i = 0; i < args.length; i++) {
                          reporter.log(args[i]);
                      }
                  }
              }
          }

          function isDetached(element) {
              function isInDocument(element) {
                  var isInShadowRoot = element.getRootNode && element.getRootNode().contains(element);
                  return element === element.ownerDocument.body || element.ownerDocument.body.contains(element) || isInShadowRoot;
              }

              if (!isInDocument(element)) {
                  return true;
              }

              // FireFox returns null style in hidden iframes. See https://github.com/wnr/element-resize-detector/issues/68 and https://bugzilla.mozilla.org/show_bug.cgi?id=795520
              if (window.getComputedStyle(element) === null) {
                  return true;
              }

              return false;
          }

          function isUnrendered(element) {
              // Check the absolute positioned container since the top level container is display: inline.
              var container = getState(element).container.childNodes[0];
              var style = window.getComputedStyle(container);
              return !style.width || style.width.indexOf("px") === -1; //Can only compute pixel value when rendered.
          }

          function getStyle() {
              // Some browsers only force layouts when actually reading the style properties of the style object, so make sure that they are all read here,
              // so that the user of the function can be sure that it will perform the layout here, instead of later (important for batching).
              var elementStyle            = window.getComputedStyle(element);
              var style                   = {};
              style.position              = elementStyle.position;
              style.width                 = element.offsetWidth;
              style.height                = element.offsetHeight;
              style.top                   = elementStyle.top;
              style.right                 = elementStyle.right;
              style.bottom                = elementStyle.bottom;
              style.left                  = elementStyle.left;
              style.widthCSS              = elementStyle.width;
              style.heightCSS             = elementStyle.height;
              return style;
          }

          function storeStartSize() {
              var style = getStyle();
              getState(element).startSize = {
                  width: style.width,
                  height: style.height
              };
              debug("Element start size", getState(element).startSize);
          }

          function initListeners() {
              getState(element).listeners = [];
          }

          function storeStyle() {
              debug("storeStyle invoked.");
              if (!getState(element)) {
                  debug("Aborting because element has been uninstalled");
                  return;
              }

              var style = getStyle();
              getState(element).style = style;
          }

          function storeCurrentSize(element, width, height) {
              getState(element).lastWidth = width;
              getState(element).lastHeight  = height;
          }

          function getExpandChildElement(element) {
              return getExpandElement(element).childNodes[0];
          }

          function getWidthOffset() {
              return 2 * scrollbarSizes.width + 1;
          }

          function getHeightOffset() {
              return 2 * scrollbarSizes.height + 1;
          }

          function getExpandWidth(width) {
              return width + 10 + getWidthOffset();
          }

          function getExpandHeight(height) {
              return height + 10 + getHeightOffset();
          }

          function getShrinkWidth(width) {
              return width * 2 + getWidthOffset();
          }

          function getShrinkHeight(height) {
              return height * 2 + getHeightOffset();
          }

          function positionScrollbars(element, width, height) {
              var expand          = getExpandElement(element);
              var shrink          = getShrinkElement(element);
              var expandWidth     = getExpandWidth(width);
              var expandHeight    = getExpandHeight(height);
              var shrinkWidth     = getShrinkWidth(width);
              var shrinkHeight    = getShrinkHeight(height);
              expand.scrollLeft   = expandWidth;
              expand.scrollTop    = expandHeight;
              shrink.scrollLeft   = shrinkWidth;
              shrink.scrollTop    = shrinkHeight;
          }

          function injectContainerElement() {
              var container = getState(element).container;

              if (!container) {
                  container                   = document.createElement("div");
                  container.className         = detectionContainerClass;
                  container.style.cssText     = buildCssTextString(["visibility: hidden", "display: inline", "width: 0px", "height: 0px", "z-index: -1", "overflow: hidden", "margin: 0", "padding: 0"]);
                  getState(element).container = container;
                  addAnimationClass(container);
                  element.appendChild(container);

                  var onAnimationStart = function () {
                      getState(element).onRendered && getState(element).onRendered();
                  };

                  addEvent(container, "animationstart", onAnimationStart);

                  // Store the event handler here so that they may be removed when uninstall is called.
                  // See uninstall function for an explanation why it is needed.
                  getState(element).onAnimationStart = onAnimationStart;
              }

              return container;
          }

          function injectScrollElements() {
              function alterPositionStyles() {
                  var style = getState(element).style;

                  if(style.position === "static") {
                      element.style.setProperty("position", "relative",options.important ? "important" : "");

                      var removeRelativeStyles = function(reporter, element, style, property) {
                          function getNumericalValue(value) {
                              return value.replace(/[^-\d\.]/g, "");
                          }

                          var value = style[property];

                          if(value !== "auto" && getNumericalValue(value) !== "0") {
                              reporter.warn("An element that is positioned static has style." + property + "=" + value + " which is ignored due to the static positioning. The element will need to be positioned relative, so the style." + property + " will be set to 0. Element: ", element);
                              element.style[property] = 0;
                          }
                      };

                      //Check so that there are no accidental styles that will make the element styled differently now that is is relative.
                      //If there are any, set them to 0 (this should be okay with the user since the style properties did nothing before [since the element was positioned static] anyway).
                      removeRelativeStyles(reporter, element, style, "top");
                      removeRelativeStyles(reporter, element, style, "right");
                      removeRelativeStyles(reporter, element, style, "bottom");
                      removeRelativeStyles(reporter, element, style, "left");
                  }
              }

              function getLeftTopBottomRightCssText(left, top, bottom, right) {
                  left = (!left ? "0" : (left + "px"));
                  top = (!top ? "0" : (top + "px"));
                  bottom = (!bottom ? "0" : (bottom + "px"));
                  right = (!right ? "0" : (right + "px"));

                  return ["left: " + left, "top: " + top, "right: " + right, "bottom: " + bottom];
              }

              debug("Injecting elements");

              if (!getState(element)) {
                  debug("Aborting because element has been uninstalled");
                  return;
              }

              alterPositionStyles();

              var rootContainer = getState(element).container;

              if (!rootContainer) {
                  rootContainer = injectContainerElement();
              }

              // Due to this WebKit bug https://bugs.webkit.org/show_bug.cgi?id=80808 (currently fixed in Blink, but still present in WebKit browsers such as Safari),
              // we need to inject two containers, one that is width/height 100% and another that is left/top -1px so that the final container always is 1x1 pixels bigger than
              // the targeted element.
              // When the bug is resolved, "containerContainer" may be removed.

              // The outer container can occasionally be less wide than the targeted when inside inline elements element in WebKit (see https://bugs.webkit.org/show_bug.cgi?id=152980).
              // This should be no problem since the inner container either way makes sure the injected scroll elements are at least 1x1 px.

              var scrollbarWidth          = scrollbarSizes.width;
              var scrollbarHeight         = scrollbarSizes.height;
              var containerContainerStyle = buildCssTextString(["position: absolute", "flex: none", "overflow: hidden", "z-index: -1", "visibility: hidden", "width: 100%", "height: 100%", "left: 0px", "top: 0px"]);
              var containerStyle          = buildCssTextString(["position: absolute", "flex: none", "overflow: hidden", "z-index: -1", "visibility: hidden"].concat(getLeftTopBottomRightCssText(-(1 + scrollbarWidth), -(1 + scrollbarHeight), -scrollbarHeight, -scrollbarWidth)));
              var expandStyle             = buildCssTextString(["position: absolute", "flex: none", "overflow: scroll", "z-index: -1", "visibility: hidden", "width: 100%", "height: 100%"]);
              var shrinkStyle             = buildCssTextString(["position: absolute", "flex: none", "overflow: scroll", "z-index: -1", "visibility: hidden", "width: 100%", "height: 100%"]);
              var expandChildStyle        = buildCssTextString(["position: absolute", "left: 0", "top: 0"]);
              var shrinkChildStyle        = buildCssTextString(["position: absolute", "width: 200%", "height: 200%"]);

              var containerContainer      = document.createElement("div");
              var container               = document.createElement("div");
              var expand                  = document.createElement("div");
              var expandChild             = document.createElement("div");
              var shrink                  = document.createElement("div");
              var shrinkChild             = document.createElement("div");

              // Some browsers choke on the resize system being rtl, so force it to ltr. https://github.com/wnr/element-resize-detector/issues/56
              // However, dir should not be set on the top level container as it alters the dimensions of the target element in some browsers.
              containerContainer.dir              = "ltr";

              containerContainer.style.cssText    = containerContainerStyle;
              containerContainer.className        = detectionContainerClass;
              container.className                 = detectionContainerClass;
              container.style.cssText             = containerStyle;
              expand.style.cssText                = expandStyle;
              expandChild.style.cssText           = expandChildStyle;
              shrink.style.cssText                = shrinkStyle;
              shrinkChild.style.cssText           = shrinkChildStyle;

              expand.appendChild(expandChild);
              shrink.appendChild(shrinkChild);
              container.appendChild(expand);
              container.appendChild(shrink);
              containerContainer.appendChild(container);
              rootContainer.appendChild(containerContainer);

              function onExpandScroll() {
                  var state = getState(element);
                  if (state && state.onExpand) {
                      state.onExpand();
                  } else {
                      debug("Aborting expand scroll handler: element has been uninstalled");
                  }
              }

              function onShrinkScroll() {
                  var state = getState(element);
                  if (state && state.onShrink) {
                      state.onShrink();
                  } else {
                      debug("Aborting shrink scroll handler: element has been uninstalled");
                  }
              }

              addEvent(expand, "scroll", onExpandScroll);
              addEvent(shrink, "scroll", onShrinkScroll);

              // Store the event handlers here so that they may be removed when uninstall is called.
              // See uninstall function for an explanation why it is needed.
              getState(element).onExpandScroll = onExpandScroll;
              getState(element).onShrinkScroll = onShrinkScroll;
          }

          function registerListenersAndPositionElements() {
              function updateChildSizes(element, width, height) {
                  var expandChild             = getExpandChildElement(element);
                  var expandWidth             = getExpandWidth(width);
                  var expandHeight            = getExpandHeight(height);
                  expandChild.style.setProperty("width", expandWidth + "px", options.important ? "important" : "");
                  expandChild.style.setProperty("height", expandHeight + "px", options.important ? "important" : "");
              }

              function updateDetectorElements(done) {
                  var width           = element.offsetWidth;
                  var height          = element.offsetHeight;

                  // Check whether the size has actually changed since last time the algorithm ran. If not, some steps may be skipped.
                  var sizeChanged = width !== getState(element).lastWidth || height !== getState(element).lastHeight;

                  debug("Storing current size", width, height);

                  // Store the size of the element sync here, so that multiple scroll events may be ignored in the event listeners.
                  // Otherwise the if-check in handleScroll is useless.
                  storeCurrentSize(element, width, height);

                  // Since we delay the processing of the batch, there is a risk that uninstall has been called before the batch gets to execute.
                  // Since there is no way to cancel the fn executions, we need to add an uninstall guard to all fns of the batch.

                  batchProcessor.add(0, function performUpdateChildSizes() {
                      if (!sizeChanged) {
                          return;
                      }

                      if (!getState(element)) {
                          debug("Aborting because element has been uninstalled");
                          return;
                      }

                      if (!areElementsInjected()) {
                          debug("Aborting because element container has not been initialized");
                          return;
                      }

                      if (options.debug) {
                          var w = element.offsetWidth;
                          var h = element.offsetHeight;

                          if (w !== width || h !== height) {
                              reporter.warn(idHandler.get(element), "Scroll: Size changed before updating detector elements.");
                          }
                      }

                      updateChildSizes(element, width, height);
                  });

                  batchProcessor.add(1, function updateScrollbars() {
                      // This function needs to be invoked event though the size is unchanged. The element could have been resized very quickly and then
                      // been restored to the original size, which will have changed the scrollbar positions.

                      if (!getState(element)) {
                          debug("Aborting because element has been uninstalled");
                          return;
                      }

                      if (!areElementsInjected()) {
                          debug("Aborting because element container has not been initialized");
                          return;
                      }

                      positionScrollbars(element, width, height);
                  });

                  if (sizeChanged && done) {
                      batchProcessor.add(2, function () {
                          if (!getState(element)) {
                              debug("Aborting because element has been uninstalled");
                              return;
                          }

                          if (!areElementsInjected()) {
                            debug("Aborting because element container has not been initialized");
                            return;
                          }

                          done();
                      });
                  }
              }

              function areElementsInjected() {
                  return !!getState(element).container;
              }

              function notifyListenersIfNeeded() {
                  function isFirstNotify() {
                      return getState(element).lastNotifiedWidth === undefined;
                  }

                  debug("notifyListenersIfNeeded invoked");

                  var state = getState(element);

                  // Don't notify if the current size is the start size, and this is the first notification.
                  if (isFirstNotify() && state.lastWidth === state.startSize.width && state.lastHeight === state.startSize.height) {
                      return debug("Not notifying: Size is the same as the start size, and there has been no notification yet.");
                  }

                  // Don't notify if the size already has been notified.
                  if (state.lastWidth === state.lastNotifiedWidth && state.lastHeight === state.lastNotifiedHeight) {
                      return debug("Not notifying: Size already notified");
                  }


                  debug("Current size not notified, notifying...");
                  state.lastNotifiedWidth = state.lastWidth;
                  state.lastNotifiedHeight = state.lastHeight;
                  forEach$1(getState(element).listeners, function (listener) {
                      listener(element);
                  });
              }

              function handleRender() {
                  debug("startanimation triggered.");

                  if (isUnrendered(element)) {
                      debug("Ignoring since element is still unrendered...");
                      return;
                  }

                  debug("Element rendered.");
                  var expand = getExpandElement(element);
                  var shrink = getShrinkElement(element);
                  if (expand.scrollLeft === 0 || expand.scrollTop === 0 || shrink.scrollLeft === 0 || shrink.scrollTop === 0) {
                      debug("Scrollbars out of sync. Updating detector elements...");
                      updateDetectorElements(notifyListenersIfNeeded);
                  }
              }

              function handleScroll() {
                  debug("Scroll detected.");

                  if (isUnrendered(element)) {
                      // Element is still unrendered. Skip this scroll event.
                      debug("Scroll event fired while unrendered. Ignoring...");
                      return;
                  }

                  updateDetectorElements(notifyListenersIfNeeded);
              }

              debug("registerListenersAndPositionElements invoked.");

              if (!getState(element)) {
                  debug("Aborting because element has been uninstalled");
                  return;
              }

              getState(element).onRendered = handleRender;
              getState(element).onExpand = handleScroll;
              getState(element).onShrink = handleScroll;

              var style = getState(element).style;
              updateChildSizes(element, style.width, style.height);
          }

          function finalizeDomMutation() {
              debug("finalizeDomMutation invoked.");

              if (!getState(element)) {
                  debug("Aborting because element has been uninstalled");
                  return;
              }

              var style = getState(element).style;
              storeCurrentSize(element, style.width, style.height);
              positionScrollbars(element, style.width, style.height);
          }

          function ready() {
              callback(element);
          }

          function install() {
              debug("Installing...");
              initListeners();
              storeStartSize();

              batchProcessor.add(0, storeStyle);
              batchProcessor.add(1, injectScrollElements);
              batchProcessor.add(2, registerListenersAndPositionElements);
              batchProcessor.add(3, finalizeDomMutation);
              batchProcessor.add(4, ready);
          }

          debug("Making detectable...");

          if (isDetached(element)) {
              debug("Element is detached");

              injectContainerElement();

              debug("Waiting until element is attached...");

              getState(element).onRendered = function () {
                  debug("Element is now attached");
                  install();
              };
          } else {
              install();
          }
      }

      function uninstall(element) {
          var state = getState(element);

          if (!state) {
              // Uninstall has been called on a non-erd element.
              return;
          }

          // Uninstall may have been called in the following scenarios:
          // (1) Right between the sync code and async batch (here state.busy = true, but nothing have been registered or injected).
          // (2) In the ready callback of the last level of the batch by another element (here, state.busy = true, but all the stuff has been injected).
          // (3) After the installation process (here, state.busy = false and all the stuff has been injected).
          // So to be on the safe side, let's check for each thing before removing.

          // We need to remove the event listeners, because otherwise the event might fire on an uninstall element which results in an error when trying to get the state of the element.
          state.onExpandScroll && removeEvent(getExpandElement(element), "scroll", state.onExpandScroll);
          state.onShrinkScroll && removeEvent(getShrinkElement(element), "scroll", state.onShrinkScroll);
          state.onAnimationStart && removeEvent(state.container, "animationstart", state.onAnimationStart);

          state.container && element.removeChild(state.container);
      }

      return {
          makeDetectable: makeDetectable,
          addListener: addListener,
          uninstall: uninstall,
          initDocument: initDocument
      };
  };

  var forEach                 = collectionUtilsExports.forEach;
  var elementUtilsMaker       = elementUtils;
  var listenerHandlerMaker    = listenerHandler;
  var idGeneratorMaker        = idGenerator;
  var idHandlerMaker          = idHandler;
  var reporterMaker           = reporter;
  var browserDetector         = browserDetectorExports;
  var batchProcessorMaker     = batchProcessor;
  var stateHandler            = stateHandler$1;

  //Detection strategies.
  var objectStrategyMaker     = object;
  var scrollStrategyMaker     = scroll;

  function isCollection(obj) {
      return Array.isArray(obj) || obj.length !== undefined;
  }

  function toArray(collection) {
      if (!Array.isArray(collection)) {
          var array = [];
          forEach(collection, function (obj) {
              array.push(obj);
          });
          return array;
      } else {
          return collection;
      }
  }

  function isElement$1(obj) {
      return obj && obj.nodeType === 1;
  }

  /**
   * @typedef idHandler
   * @type {object}
   * @property {function} get Gets the resize detector id of the element.
   * @property {function} set Generate and sets the resize detector id of the element.
   */

  /**
   * @typedef Options
   * @type {object}
   * @property {boolean} callOnAdd    Determines if listeners should be called when they are getting added.
                                      Default is true. If true, the listener is guaranteed to be called when it has been added.
                                      If false, the listener will not be guarenteed to be called when it has been added (does not prevent it from being called).
   * @property {idHandler} idHandler  A custom id handler that is responsible for generating, setting and retrieving id's for elements.
                                      If not provided, a default id handler will be used.
   * @property {reporter} reporter    A custom reporter that handles reporting logs, warnings and errors.
                                      If not provided, a default id handler will be used.
                                      If set to false, then nothing will be reported.
   * @property {boolean} debug        If set to true, the the system will report debug messages as default for the listenTo method.
   */

  /**
   * Creates an element resize detector instance.
   * @public
   * @param {Options?} options Optional global options object that will decide how this instance will work.
   */
  var elementResizeDetector = function(options) {
      options = options || {};

      //idHandler is currently not an option to the listenTo function, so it should not be added to globalOptions.
      var idHandler;

      if (options.idHandler) {
          // To maintain compatability with idHandler.get(element, readonly), make sure to wrap the given idHandler
          // so that readonly flag always is true when it's used here. This may be removed next major version bump.
          idHandler = {
              get: function (element) { return options.idHandler.get(element, true); },
              set: options.idHandler.set
          };
      } else {
          var idGenerator = idGeneratorMaker();
          var defaultIdHandler = idHandlerMaker({
              idGenerator: idGenerator,
              stateHandler: stateHandler
          });
          idHandler = defaultIdHandler;
      }

      //reporter is currently not an option to the listenTo function, so it should not be added to globalOptions.
      var reporter = options.reporter;

      if(!reporter) {
          //If options.reporter is false, then the reporter should be quiet.
          var quiet = reporter === false;
          reporter = reporterMaker(quiet);
      }

      //batchProcessor is currently not an option to the listenTo function, so it should not be added to globalOptions.
      var batchProcessor = getOption(options, "batchProcessor", batchProcessorMaker({ reporter: reporter }));

      //Options to be used as default for the listenTo function.
      var globalOptions = {};
      globalOptions.callOnAdd     = !!getOption(options, "callOnAdd", true);
      globalOptions.debug         = !!getOption(options, "debug", false);

      var eventListenerHandler    = listenerHandlerMaker(idHandler);
      var elementUtils            = elementUtilsMaker({
          stateHandler: stateHandler
      });

      //The detection strategy to be used.
      var detectionStrategy;
      var desiredStrategy = getOption(options, "strategy", "object");
      var importantCssRules = getOption(options, "important", false);
      var strategyOptions = {
          reporter: reporter,
          batchProcessor: batchProcessor,
          stateHandler: stateHandler,
          idHandler: idHandler,
          important: importantCssRules
      };

      if(desiredStrategy === "scroll") {
          if (browserDetector.isLegacyOpera()) {
              reporter.warn("Scroll strategy is not supported on legacy Opera. Changing to object strategy.");
              desiredStrategy = "object";
          } else if (browserDetector.isIE(9)) {
              reporter.warn("Scroll strategy is not supported on IE9. Changing to object strategy.");
              desiredStrategy = "object";
          }
      }

      if(desiredStrategy === "scroll") {
          detectionStrategy = scrollStrategyMaker(strategyOptions);
      } else if(desiredStrategy === "object") {
          detectionStrategy = objectStrategyMaker(strategyOptions);
      } else {
          throw new Error("Invalid strategy name: " + desiredStrategy);
      }

      //Calls can be made to listenTo with elements that are still being installed.
      //Also, same elements can occur in the elements list in the listenTo function.
      //With this map, the ready callbacks can be synchronized between the calls
      //so that the ready callback can always be called when an element is ready - even if
      //it wasn't installed from the function itself.
      var onReadyCallbacks = {};

      /**
       * Makes the given elements resize-detectable and starts listening to resize events on the elements. Calls the event callback for each event for each element.
       * @public
       * @param {Options?} options Optional options object. These options will override the global options. Some options may not be overriden, such as idHandler.
       * @param {element[]|element} elements The given array of elements to detect resize events of. Single element is also valid.
       * @param {function} listener The callback to be executed for each resize event for each element.
       */
      function listenTo(options, elements, listener) {
          function onResizeCallback(element) {
              var listeners = eventListenerHandler.get(element);
              forEach(listeners, function callListenerProxy(listener) {
                  listener(element);
              });
          }

          function addListener(callOnAdd, element, listener) {
              eventListenerHandler.add(element, listener);

              if(callOnAdd) {
                  listener(element);
              }
          }

          //Options object may be omitted.
          if(!listener) {
              listener = elements;
              elements = options;
              options = {};
          }

          if(!elements) {
              throw new Error("At least one element required.");
          }

          if(!listener) {
              throw new Error("Listener required.");
          }

          if (isElement$1(elements)) {
              // A single element has been passed in.
              elements = [elements];
          } else if (isCollection(elements)) {
              // Convert collection to array for plugins.
              // TODO: May want to check so that all the elements in the collection are valid elements.
              elements = toArray(elements);
          } else {
              return reporter.error("Invalid arguments. Must be a DOM element or a collection of DOM elements.");
          }

          var elementsReady = 0;

          var callOnAdd = getOption(options, "callOnAdd", globalOptions.callOnAdd);
          var onReadyCallback = getOption(options, "onReady", function noop() {});
          var debug = getOption(options, "debug", globalOptions.debug);

          forEach(elements, function attachListenerToElement(element) {
              if (!stateHandler.getState(element)) {
                  stateHandler.initState(element);
                  idHandler.set(element);
              }

              var id = idHandler.get(element);

              debug && reporter.log("Attaching listener to element", id, element);

              if(!elementUtils.isDetectable(element)) {
                  debug && reporter.log(id, "Not detectable.");
                  if(elementUtils.isBusy(element)) {
                      debug && reporter.log(id, "System busy making it detectable");

                      //The element is being prepared to be detectable. Do not make it detectable.
                      //Just add the listener, because the element will soon be detectable.
                      addListener(callOnAdd, element, listener);
                      onReadyCallbacks[id] = onReadyCallbacks[id] || [];
                      onReadyCallbacks[id].push(function onReady() {
                          elementsReady++;

                          if(elementsReady === elements.length) {
                              onReadyCallback();
                          }
                      });
                      return;
                  }

                  debug && reporter.log(id, "Making detectable...");
                  //The element is not prepared to be detectable, so do prepare it and add a listener to it.
                  elementUtils.markBusy(element, true);
                  return detectionStrategy.makeDetectable({ debug: debug, important: importantCssRules }, element, function onElementDetectable(element) {
                      debug && reporter.log(id, "onElementDetectable");

                      if (stateHandler.getState(element)) {
                          elementUtils.markAsDetectable(element);
                          elementUtils.markBusy(element, false);
                          detectionStrategy.addListener(element, onResizeCallback);
                          addListener(callOnAdd, element, listener);

                          // Since the element size might have changed since the call to "listenTo", we need to check for this change,
                          // so that a resize event may be emitted.
                          // Having the startSize object is optional (since it does not make sense in some cases such as unrendered elements), so check for its existance before.
                          // Also, check the state existance before since the element may have been uninstalled in the installation process.
                          var state = stateHandler.getState(element);
                          if (state && state.startSize) {
                              var width = element.offsetWidth;
                              var height = element.offsetHeight;
                              if (state.startSize.width !== width || state.startSize.height !== height) {
                                  onResizeCallback(element);
                              }
                          }

                          if(onReadyCallbacks[id]) {
                              forEach(onReadyCallbacks[id], function(callback) {
                                  callback();
                              });
                          }
                      } else {
                          // The element has been unisntalled before being detectable.
                          debug && reporter.log(id, "Element uninstalled before being detectable.");
                      }

                      delete onReadyCallbacks[id];

                      elementsReady++;
                      if(elementsReady === elements.length) {
                          onReadyCallback();
                      }
                  });
              }

              debug && reporter.log(id, "Already detecable, adding listener.");

              //The element has been prepared to be detectable and is ready to be listened to.
              addListener(callOnAdd, element, listener);
              elementsReady++;
          });

          if(elementsReady === elements.length) {
              onReadyCallback();
          }
      }

      function uninstall(elements) {
          if(!elements) {
              return reporter.error("At least one element is required.");
          }

          if (isElement$1(elements)) {
              // A single element has been passed in.
              elements = [elements];
          } else if (isCollection(elements)) {
              // Convert collection to array for plugins.
              // TODO: May want to check so that all the elements in the collection are valid elements.
              elements = toArray(elements);
          } else {
              return reporter.error("Invalid arguments. Must be a DOM element or a collection of DOM elements.");
          }

          forEach(elements, function (element) {
              eventListenerHandler.removeAllListeners(element);
              detectionStrategy.uninstall(element);
              stateHandler.cleanState(element);
          });
      }

      function initDocument(targetDocument) {
          detectionStrategy.initDocument && detectionStrategy.initDocument(targetDocument);
      }

      return {
          listenTo: listenTo,
          removeListener: eventListenerHandler.removeListener,
          removeAllListeners: eventListenerHandler.removeAllListeners,
          uninstall: uninstall,
          initDocument: initDocument
      };
  };

  function getOption(options, name, defaultValue) {
      var value = options[name];

      if((value === undefined || value === null) && defaultValue !== undefined) {
          return defaultValue;
      }

      return value;
  }

  var elementResizeDetectorMaker = /*@__PURE__*/getDefaultExportFromCjs(elementResizeDetector);

  var getResizeDetectorIns = (function () {
    var resizerIns = elementResizeDetectorMaker({
      strategy: 'scroll',
      callOnAdd: true
    });
    var getRaf = function getRaf() {
      var w = window;
      return w.requestAnimationFrame || w.webkitRequestAnimationFrame || w.mozRequestAnimationFrame || w.oRequestAnimationFrame || w.msRequestAnimationFrame || function (callback) {
        w.setTimeout(callback, 1000 / 60);
      };
    };
    resizerIns.debncListenTo = function (el, callback) {
      return resizerIns.listenTo(el, function (func, wait) {
        var timeout;
        return function () {
          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }
          var later = function later() {
            clearTimeout(timeout);
            getRaf()(function () {
              return func.apply(void 0, args);
            });
          };
          clearTimeout(timeout);
          timeout = setTimeout(later, wait);
        };
      }(callback, 10));
    };
    return resizerIns;
  });

  function El(_ref) {
    var el = _ref.el,
      sizeDimension = _ref.sizeDimension,
      sizeDirections = _ref.sizeDirections;
    this._el = el;
    /**would be width|height */
    this._sizeDimension = sizeDimension;
    /**would be "right"|"left"|"top"|"bottom" */
    this._sizeDirections = sizeDirections;
  }
  El.prototype = {
    getStyle: function getStyle() {
      return this._style = this._style || this._el.currentStyle || window.getComputedStyle(this._el);
    },
    getPos: function getPos() {
      return this._pos = this._pos || this._el.getBoundingClientRect();
    },
    /**
     * including padding, border and margin
     * @param {"Right"|"Bottom"|"Left"|"Top"} dir
     * @returns {Number}
     */
    getSpacing: function getSpacing(dir) {
      var style = this.getStyle();
      return parseFloat(style["margin" + dir]) + parseFloat(style["padding" + dir]) + parseFloat(style["border" + dir + "Width"]);
    },
    /**not include padding and borders */
    getSize: function getSize() {
      return this.size = this.size || function (that) {
        var style = that.getStyle();
        return that.getPos()[that._sizeDimension] - parseFloat(style["padding" + that._sizeDirections[0]]) - parseFloat(style["padding" + that._sizeDirections[1]]) - parseFloat(style["border" + that._sizeDirections[1] + "Width"]) - parseFloat(style["border" + that._sizeDirections[0] + "Width"]);
      }(this);
    },
    /**include margin */
    getFullSize: function getFullSize() {
      return this.fullSize = this.fullSize || function (that) {
        var style = that.getStyle();
        return that.getPos()[that._sizeDimension] + parseFloat(style["margin" + that._sizeDirections[0]]) + parseFloat(style["margin" + that._sizeDirections[1]]);
      }(this);
    }
  };

  function Els(deps, op) {
    this._geElInstance = deps.geElInstance;
    this.els = new Map();
    this._sizeFactory(op.isVertical);
  }
  Els.prototype = {
    _sizeFactory: function _sizeFactory(isVertical) {
      if (isVertical === true) {
        this._sizeDimension = 'height';
        this._sizeDirections = ['Top', 'Bottom'];
      } else {
        this._sizeDimension = 'width';
        this._sizeDirections = ['Left', 'Right'];
      }
    },
    getEl: function getEl(el) {
      this.els.has(el) || this.els.set(el, this._geElInstance({
        el: el,
        sizeDimension: this._sizeDimension,
        sizeDirections: this._sizeDirections
      }));
      return this.els.get(el);
    }
  };

  function ElsFactory () {
    return Function.prototype.bind.apply(Els, [this || null, {
      geElInstance: function geElInstance(op) {
        return new El(op);
      }
    }].concat(Array.prototype.slice.call(arguments)))();
  }

  var constructor = function DistanceFrom(deps, _ref) {
    var baseEl = _ref.baseEl,
      isVertical = _ref.isVertical,
      dir = _ref.dir;
    deps.Base.call(this, {
      isVertical: isVertical
    });
    this.baseEl = baseEl;
    this._distanceFactory(isVertical, dir);
  };
  var methods = {
    _distanceFactory: function _distanceFactory(isVertical, dir) {
      if (isVertical == true) this.getDistance = this._getVerticalDistance;else if (dir === 'ltr') this.getDistance = this._getLtrDistance;else this.getDistance = this._getRtlDistance;
      return this;
    },
    _getVerticalDistance: function _getVerticalDistance(el) {
      var baseElIns = this.getEl(this.baseEl);
      return this._getResult(baseElIns.getPos().bottom - this.getEl(el).getPos().bottom - parseFloat(baseElIns.getStyle().paddingBottom));
    },
    _getRtlDistance: function _getRtlDistance(el) {
      var baseElIns = this.getEl(this.baseEl);
      return this._getResult(this.getEl(el).getPos().left - baseElIns.getPos().left - parseFloat(baseElIns.getStyle().paddingLeft));
    },
    _getLtrDistance: function _getLtrDistance(el) {
      var baseElIns = this.getEl(this.baseEl);
      return this._getResult(baseElIns.getPos().right - this.getEl(el).getPos().right - parseFloat(baseElIns.getStyle().paddingRight));
    },
    _getResult: function _getResult(value) {
      var obj = Object.create({
        sub: function sub(value) {
          this.value = this.value - value;
          return this;
        }
      });
      obj.value = value;
      return obj;
    }
  };

  constructor.prototype = Object.create(Els.prototype);
  Object.assign(constructor.prototype, methods).constructor = constructor;
  var ElManagement = constructor.bind(null, {
    Base: function Base() {
      ElsFactory.apply(this, arguments);
    }
  });

  /**
   * @class Api
   * @param {Object} options
   * @param {HTMLElement} options.containerElement - the first ancestor of the Tablist element which its width|height is not relative to tabList element but is relative to viewport
   * @param {HTMLElement} options.buttonElement - should be sibling element of Tablist element
   * @param {HTMLElement} options.tablistElement
   * @param {String} [options.tabDisplay="inline-flex"] - default value is "inline-flex". would be display of tab elements
   * @param {String} [options.containerDisplay="flex"] - default value is "flex". would be display of container element
   */
  var Api$1 = function Api() {
    var arg = arguments;
    this._setOptions(arg[1]);
    this._tablistEl = null;
    var _arg$ = arg[0](),
      getElManagementIns = _arg$.getElManagementIns;
    this._getElManagementIns = getElManagementIns;
    this._tabs = null;
    this._tabsCount = null;
    this._setEls();
  };
  Api$1.prototype = {
    _setOptions: function _setOptions(options) {
      this._options = Object.assign({}, {
        containerDisplay: 'flex',
        tabDisplay: 'inline-flex',
        buttonElement: null,
        containerElement: null,
        tablistElement: null
      }, options);
    },
    _setEls: function _setEls() {
      this._tablistEl = this._options.tablistElement;
      this._tablistEl.style.overflow = 'visible';
      this._options.containerElement.style.overflow = 'hidden';
      this._options.containerElement.style.whiteSpace = 'nowrap';
      return this;
    },
    _showBtn: function _showBtn() {
      this._options.buttonElement.style.opacity = 1;
      this._options.buttonElement.style.position = 'relative';
      this._options.buttonElement.style.pointerEvents = 'all';
    },
    _hideBtn: function _hideBtn() {
      this._options.buttonElement.style.opacity = 0;
      this._options.buttonElement.style.position = 'absolute';
      this._options.buttonElement.style.pointerEvents = 'none';
    },
    _checkOverflow: function _checkOverflow(lastTab) {
      return this.els.getDistance(lastTab).value < 0;
    },
    _showAll: function _showAll() {
      this._options.containerElement.style.display = 'none';
      var tabDisplay = this._options.tabDisplay;
      for (var i = 0, tabs = this._tablistEl.children, tabsCount = tabs.length; i < tabsCount; i++) {
        tabs[i].style.display = tabDisplay;
      }
      this._hideBtn();
      this._options.containerElement.style.display = this._options.containerDisplay;
    },
    _hideTabs: function _hideTabs(firstHiddenTabIndex, selectedTabInfo, includeSelectedTab) {
      var hiddenTabs = [];
      this._options.containerElement.style.display = 'none';
      var selectedTabIndex = selectedTabInfo.index;
      for (var i = firstHiddenTabIndex, tabsCount = this._tabsCount; i < tabsCount; i++) {
        if (includeSelectedTab || i !== selectedTabIndex) {
          this._tabs[i].style.display = 'none';
          hiddenTabs.push({
            el: this._tabs[i],
            index: i
          });
        }
      }
      this._showBtn();
      this._options.containerElement.style.display = this._options.containerDisplay;
      return hiddenTabs;
    },
    _getSelectedTabInfo: function _getSelectedTabInfo(tabs, selectedTabIndex) {
      var index = selectedTabIndex;
      var el = index >= 0 ? tabs[index] : null;
      var overflow = el ? this.els.getDistance(el).sub(this.els.getEl(this._options.tablistElement).getSpacing(this._btnPositionRelativeToTablist)).sub(this.els.getEl(this._options.buttonElement).getFullSize()).value <= 0 : false;
      var overflowFullSize = overflow ? this.els.getEl(el).getFullSize() : 0;
      return {
        index: index,
        overflowFullSize: overflowFullSize
      };
    },
    _validateTabsCount: function _validateTabsCount() {
      this._tabs = this._tablistEl.children;
      this._tabsCount = this._tabs.length;
      return this._tabsCount ? true : false;
    },
    /**
     *
     * @param {Number} selectedTabIndex
     * @param {"ltr"|"rtl"} [direction="ltr"]
     * @param {Boolean} [isVertical=false]
     * @returns {Array.<{el: HTMLElement , index: Number}>}
     */
    resize: function resize(selectedTabIndex, direction, isVertical) {
      if (selectedTabIndex === void 0) {
        selectedTabIndex = '';
      }
      if (direction === void 0) {
        direction = 'ltr';
      }
      if (isVertical === void 0) {
        isVertical = false;
      }
      if (this._validateTabsCount() === false) {
        return [];
      }
      this._showAll();
      this.els = this._getElManagementIns({
        baseEl: this._options.containerElement,
        isVertical: isVertical,
        dir: direction
      });
      var _lastTab = this._tabs[this._tabsCount - 1];
      if (this._checkOverflow(_lastTab) === false) {
        return [];
      }
      var selectedTabInfo = this._setBtnPositionRelativeToTablist(isVertical, direction)._getSelectedTabInfo(this._tabs, selectedTabIndex);
      return this._validateSliderMinSize(selectedTabInfo) ? this._hideTabs(this._findFirstHiddenTabIndexFactory(selectedTabInfo, this._getSearchBoundries(selectedTabInfo), this._getOrder(this._tabs[0], _lastTab)), selectedTabInfo) : this._hideTabs(0, selectedTabInfo, true);
    },
    _validateSliderMinSize: function _validateSliderMinSize(selectedTabInfo) {
      return this.els.getEl(this._options.tablistElement).getSpacing(this._btnReversePositionRelativeToTablist) + selectedTabInfo.overflowFullSize + this.els.getEl(this._options.tablistElement).getSpacing(this._btnPositionRelativeToTablist) + this.els.getEl(this._options.buttonElement).getFullSize() >= this.els.getEl(this._options.containerElement).getSize() ? false : true;
    },
    _getOrder: function _getOrder(firstTab, lastTab) {
      return Math.abs(this.els.getDistance(lastTab).value) > Math.abs(this.els.getDistance(firstTab).value) ? 'asc' : 'desc';
    },
    _getSearchBoundries: function _getSearchBoundries(selectedTabInfo) {
      var overflowFullSize = selectedTabInfo.overflowFullSize,
        pivotIndex = selectedTabInfo.index;
      //if selected tab is not existed
      if (pivotIndex < 0) {
        return [0, this._tabsCount - 2];
      }
      var isSelectedTabOverflow = overflowFullSize > 0;
      return isSelectedTabOverflow ? [0, pivotIndex - 1] : [pivotIndex + 1, this._tabsCount - 2];
    },
    _getTabDis: function _getTabDis(selectedTabInfo, el) {
      return this.els.getDistance(el).sub(selectedTabInfo.overflowFullSize).sub(this.els.getEl(this._options.tablistElement).getSpacing(this._btnPositionRelativeToTablist)).sub(this.els.getEl(this._options.buttonElement).getFullSize());
    },
    _findFirstHiddenTabIndexDSCE: function _findFirstHiddenTabIndexDSCE(selectedTabInfo, start, stop) {
      var value = this._tabsCount - 1;
      for (var i = stop; i >= start; i--) {
        if (this._getTabDis(selectedTabInfo, this._tabs[i]).value <= 0) {
          value = i;
        } else {
          break;
        }
      }
      return value;
    },
    _findFirstHiddenTabIndexASC: function _findFirstHiddenTabIndexASC(selectedTabInfo, start, stop) {
      for (var i = start; i <= stop; i++) {
        if (this._getTabDis(selectedTabInfo, this._tabs[i]).value <= 0) {
          return i;
        }
      }
      return this._tabsCount - 1;
    },
    _findFirstHiddenTabIndexFactory: function _findFirstHiddenTabIndexFactory(selectedTabInfo, _ref, order) {
      var start = _ref[0],
        stop = _ref[1];
      return order === 'asc' ? this._findFirstHiddenTabIndexASC(selectedTabInfo, start, stop) : this._findFirstHiddenTabIndexDSCE(selectedTabInfo, start, stop);
    },
    /** set _btnPositionRelativeToTablist and _btnReversePositionRelativeToTablist */
    _setBtnPositionRelativeToTablist: function _setBtnPositionRelativeToTablist(isVertical, dir) {
      if (!this._btnPositionRelativeToTablist) {
        var pos = '',
          reversePos = '';
        if (isVertical == true) {
          pos = 'Bottom'; //the Button is rendered below the Tablist
          reversePos = 'Top';
        } else {
          if (dir === 'ltr') {
            pos = 'Right'; //the Button is rendered in left-side of the Tablist
            reversePos = 'Left';
          } else {
            pos = 'Left'; //the Button is rendered in right-side of the Tablist
            reversePos = 'Right';
          }
        }
        /**
         * specifics where Button is relative to Tablist
         * @type {"Right"|"Bottom"|"Left"}
         */
        this._btnPositionRelativeToTablist = pos;
        /**
         * reverse Button position relative to Tablist
         * @type {"Right"|"Top"|"Left"}
         */
        this._btnReversePositionRelativeToTablist = reversePos;
      }
      return this;
    }
  };

  /**
   * @type {typeof Api}
   */
  var tabsMoreButton = Api$1.bind(undefined, function () {
    return {
      getElManagementIns: function getElManagementIns(params) {
        return new ElManagement(params);
      }
    };
  });

  function ShowMoreTabs$1(getDeps, props) {
    var ctx = props.ctx,
      openTabIDs = props.openTabIDs,
      selectedTabID = props.selectedTabID;
    var _useState = React.useState(''),
      hiddenTabIDs = _useState[0],
      setHiddenTabIDs = _useState[1];
    var _getDeps = getDeps(),
      getInstance = _getDeps.getInstance,
      getResizeDetectorIns = _getDeps.getResizeDetectorIns;
    var ref = React.useRef();
    ref.current = ref.current || {
      ins: getInstance(ctx, setHiddenTabIDs)
    };
    var ins = ref.current.ins;
    var openTabIDsString = openTabIDs.toString();
    React.useLayoutEffect(function () {
      ins.setResizer();
    }, []);
    React.useLayoutEffect(function () {
      var resizeDetectorIns = getResizeDetectorIns();
      ins.installResizer(resizeDetectorIns);
      return function () {
        ins.uninstallResizer(resizeDetectorIns);
      };
    }, []);
    React.useLayoutEffect(function () {
      ins.resize();
    }, [openTabIDsString, selectedTabID]);
    var ButtonComponent = ctx.getOption('moreButtonPlugin_buttonComponent');
    return /*#__PURE__*/React.createElement("div", {
      ref: ins.btnRef,
      className: ctx.getSetting('tabClass') + ' ' + ctx.getSetting('showMoreContainerClass')
    }, /*#__PURE__*/React.createElement(ButtonComponent, {
      hiddenTabIDs: hiddenTabIDs,
      instance: ctx.userProxy
    }));
  }

  var getDeps$1 = function getDeps() {
    return {
      getResizeDetectorIns: getResizeDetectorIns,
      getInstance: function getInstance(ctx, setHiddenTabIDs) {
        return new Api$2({
          setHiddenTabIDs: setHiddenTabIDs,
          btnRef: /*#__PURE__*/React.createRef(),
          getResizerIns: function getResizerIns(options) {
            return new (Function.prototype.bind.apply(tabsMoreButton))(options);
          },
          ctx: ctx
        });
      }
    };
  };
  var ShowMoreTabs = ShowMoreTabs$1.bind(null, getDeps$1);

  function getWindow(node) {
    if (node == null) {
      return window;
    }

    if (node.toString() !== '[object Window]') {
      var ownerDocument = node.ownerDocument;
      return ownerDocument ? ownerDocument.defaultView || window : window;
    }

    return node;
  }

  function isElement(node) {
    var OwnElement = getWindow(node).Element;
    return node instanceof OwnElement || node instanceof Element;
  }

  function isHTMLElement(node) {
    var OwnElement = getWindow(node).HTMLElement;
    return node instanceof OwnElement || node instanceof HTMLElement;
  }

  function isShadowRoot(node) {
    // IE 11 has no ShadowRoot
    if (typeof ShadowRoot === 'undefined') {
      return false;
    }

    var OwnElement = getWindow(node).ShadowRoot;
    return node instanceof OwnElement || node instanceof ShadowRoot;
  }

  var max = Math.max;
  var min = Math.min;
  var round = Math.round;

  function getUAString() {
    var uaData = navigator.userAgentData;

    if (uaData != null && uaData.brands && Array.isArray(uaData.brands)) {
      return uaData.brands.map(function (item) {
        return item.brand + "/" + item.version;
      }).join(' ');
    }

    return navigator.userAgent;
  }

  function isLayoutViewport() {
    return !/^((?!chrome|android).)*safari/i.test(getUAString());
  }

  function getBoundingClientRect(element, includeScale, isFixedStrategy) {
    if (includeScale === void 0) {
      includeScale = false;
    }

    if (isFixedStrategy === void 0) {
      isFixedStrategy = false;
    }

    var clientRect = element.getBoundingClientRect();
    var scaleX = 1;
    var scaleY = 1;

    if (includeScale && isHTMLElement(element)) {
      scaleX = element.offsetWidth > 0 ? round(clientRect.width) / element.offsetWidth || 1 : 1;
      scaleY = element.offsetHeight > 0 ? round(clientRect.height) / element.offsetHeight || 1 : 1;
    }

    var _ref = isElement(element) ? getWindow(element) : window,
        visualViewport = _ref.visualViewport;

    var addVisualOffsets = !isLayoutViewport() && isFixedStrategy;
    var x = (clientRect.left + (addVisualOffsets && visualViewport ? visualViewport.offsetLeft : 0)) / scaleX;
    var y = (clientRect.top + (addVisualOffsets && visualViewport ? visualViewport.offsetTop : 0)) / scaleY;
    var width = clientRect.width / scaleX;
    var height = clientRect.height / scaleY;
    return {
      width: width,
      height: height,
      top: y,
      right: x + width,
      bottom: y + height,
      left: x,
      x: x,
      y: y
    };
  }

  function getWindowScroll(node) {
    var win = getWindow(node);
    var scrollLeft = win.pageXOffset;
    var scrollTop = win.pageYOffset;
    return {
      scrollLeft: scrollLeft,
      scrollTop: scrollTop
    };
  }

  function getHTMLElementScroll(element) {
    return {
      scrollLeft: element.scrollLeft,
      scrollTop: element.scrollTop
    };
  }

  function getNodeScroll(node) {
    if (node === getWindow(node) || !isHTMLElement(node)) {
      return getWindowScroll(node);
    } else {
      return getHTMLElementScroll(node);
    }
  }

  function getNodeName(element) {
    return element ? (element.nodeName || '').toLowerCase() : null;
  }

  function getDocumentElement(element) {
    // $FlowFixMe[incompatible-return]: assume body is always available
    return ((isElement(element) ? element.ownerDocument : // $FlowFixMe[prop-missing]
    element.document) || window.document).documentElement;
  }

  function getWindowScrollBarX(element) {
    // If <html> has a CSS width greater than the viewport, then this will be
    // incorrect for RTL.
    // Popper 1 is broken in this case and never had a bug report so let's assume
    // it's not an issue. I don't think anyone ever specifies width on <html>
    // anyway.
    // Browsers where the left scrollbar doesn't cause an issue report `0` for
    // this (e.g. Edge 2019, IE11, Safari)
    return getBoundingClientRect(getDocumentElement(element)).left + getWindowScroll(element).scrollLeft;
  }

  function getComputedStyle(element) {
    return getWindow(element).getComputedStyle(element);
  }

  function isScrollParent(element) {
    // Firefox wants us to check `-x` and `-y` variations as well
    var _getComputedStyle = getComputedStyle(element),
        overflow = _getComputedStyle.overflow,
        overflowX = _getComputedStyle.overflowX,
        overflowY = _getComputedStyle.overflowY;

    return /auto|scroll|overlay|hidden/.test(overflow + overflowY + overflowX);
  }

  function isElementScaled(element) {
    var rect = element.getBoundingClientRect();
    var scaleX = round(rect.width) / element.offsetWidth || 1;
    var scaleY = round(rect.height) / element.offsetHeight || 1;
    return scaleX !== 1 || scaleY !== 1;
  } // Returns the composite rect of an element relative to its offsetParent.
  // Composite means it takes into account transforms as well as layout.


  function getCompositeRect(elementOrVirtualElement, offsetParent, isFixed) {
    if (isFixed === void 0) {
      isFixed = false;
    }

    var isOffsetParentAnElement = isHTMLElement(offsetParent);
    var offsetParentIsScaled = isHTMLElement(offsetParent) && isElementScaled(offsetParent);
    var documentElement = getDocumentElement(offsetParent);
    var rect = getBoundingClientRect(elementOrVirtualElement, offsetParentIsScaled, isFixed);
    var scroll = {
      scrollLeft: 0,
      scrollTop: 0
    };
    var offsets = {
      x: 0,
      y: 0
    };

    if (isOffsetParentAnElement || !isOffsetParentAnElement && !isFixed) {
      if (getNodeName(offsetParent) !== 'body' || // https://github.com/popperjs/popper-core/issues/1078
      isScrollParent(documentElement)) {
        scroll = getNodeScroll(offsetParent);
      }

      if (isHTMLElement(offsetParent)) {
        offsets = getBoundingClientRect(offsetParent, true);
        offsets.x += offsetParent.clientLeft;
        offsets.y += offsetParent.clientTop;
      } else if (documentElement) {
        offsets.x = getWindowScrollBarX(documentElement);
      }
    }

    return {
      x: rect.left + scroll.scrollLeft - offsets.x,
      y: rect.top + scroll.scrollTop - offsets.y,
      width: rect.width,
      height: rect.height
    };
  }

  // means it doesn't take into account transforms.

  function getLayoutRect(element) {
    var clientRect = getBoundingClientRect(element); // Use the clientRect sizes if it's not been transformed.
    // Fixes https://github.com/popperjs/popper-core/issues/1223

    var width = element.offsetWidth;
    var height = element.offsetHeight;

    if (Math.abs(clientRect.width - width) <= 1) {
      width = clientRect.width;
    }

    if (Math.abs(clientRect.height - height) <= 1) {
      height = clientRect.height;
    }

    return {
      x: element.offsetLeft,
      y: element.offsetTop,
      width: width,
      height: height
    };
  }

  function getParentNode(element) {
    if (getNodeName(element) === 'html') {
      return element;
    }

    return (// this is a quicker (but less type safe) way to save quite some bytes from the bundle
      // $FlowFixMe[incompatible-return]
      // $FlowFixMe[prop-missing]
      element.assignedSlot || // step into the shadow DOM of the parent of a slotted node
      element.parentNode || ( // DOM Element detected
      isShadowRoot(element) ? element.host : null) || // ShadowRoot detected
      // $FlowFixMe[incompatible-call]: HTMLElement is a Node
      getDocumentElement(element) // fallback

    );
  }

  function getScrollParent(node) {
    if (['html', 'body', '#document'].indexOf(getNodeName(node)) >= 0) {
      // $FlowFixMe[incompatible-return]: assume body is always available
      return node.ownerDocument.body;
    }

    if (isHTMLElement(node) && isScrollParent(node)) {
      return node;
    }

    return getScrollParent(getParentNode(node));
  }

  /*
  given a DOM element, return the list of all scroll parents, up the list of ancesors
  until we get to the top window object. This list is what we attach scroll listeners
  to, because if any of these parent elements scroll, we'll need to re-calculate the
  reference element's position.
  */

  function listScrollParents(element, list) {
    var _element$ownerDocumen;

    if (list === void 0) {
      list = [];
    }

    var scrollParent = getScrollParent(element);
    var isBody = scrollParent === ((_element$ownerDocumen = element.ownerDocument) == null ? void 0 : _element$ownerDocumen.body);
    var win = getWindow(scrollParent);
    var target = isBody ? [win].concat(win.visualViewport || [], isScrollParent(scrollParent) ? scrollParent : []) : scrollParent;
    var updatedList = list.concat(target);
    return isBody ? updatedList : // $FlowFixMe[incompatible-call]: isBody tells us target will be an HTMLElement here
    updatedList.concat(listScrollParents(getParentNode(target)));
  }

  function isTableElement(element) {
    return ['table', 'td', 'th'].indexOf(getNodeName(element)) >= 0;
  }

  function getTrueOffsetParent(element) {
    if (!isHTMLElement(element) || // https://github.com/popperjs/popper-core/issues/837
    getComputedStyle(element).position === 'fixed') {
      return null;
    }

    return element.offsetParent;
  } // `.offsetParent` reports `null` for fixed elements, while absolute elements
  // return the containing block


  function getContainingBlock(element) {
    var isFirefox = /firefox/i.test(getUAString());
    var isIE = /Trident/i.test(getUAString());

    if (isIE && isHTMLElement(element)) {
      // In IE 9, 10 and 11 fixed elements containing block is always established by the viewport
      var elementCss = getComputedStyle(element);

      if (elementCss.position === 'fixed') {
        return null;
      }
    }

    var currentNode = getParentNode(element);

    if (isShadowRoot(currentNode)) {
      currentNode = currentNode.host;
    }

    while (isHTMLElement(currentNode) && ['html', 'body'].indexOf(getNodeName(currentNode)) < 0) {
      var css = getComputedStyle(currentNode); // This is non-exhaustive but covers the most common CSS properties that
      // create a containing block.
      // https://developer.mozilla.org/en-US/docs/Web/CSS/Containing_block#identifying_the_containing_block

      if (css.transform !== 'none' || css.perspective !== 'none' || css.contain === 'paint' || ['transform', 'perspective'].indexOf(css.willChange) !== -1 || isFirefox && css.willChange === 'filter' || isFirefox && css.filter && css.filter !== 'none') {
        return currentNode;
      } else {
        currentNode = currentNode.parentNode;
      }
    }

    return null;
  } // Gets the closest ancestor positioned element. Handles some edge cases,
  // such as table ancestors and cross browser bugs.


  function getOffsetParent(element) {
    var window = getWindow(element);
    var offsetParent = getTrueOffsetParent(element);

    while (offsetParent && isTableElement(offsetParent) && getComputedStyle(offsetParent).position === 'static') {
      offsetParent = getTrueOffsetParent(offsetParent);
    }

    if (offsetParent && (getNodeName(offsetParent) === 'html' || getNodeName(offsetParent) === 'body' && getComputedStyle(offsetParent).position === 'static')) {
      return window;
    }

    return offsetParent || getContainingBlock(element) || window;
  }

  var top = 'top';
  var bottom = 'bottom';
  var right = 'right';
  var left = 'left';
  var auto = 'auto';
  var basePlacements = [top, bottom, right, left];
  var start = 'start';
  var end = 'end';
  var clippingParents = 'clippingParents';
  var viewport = 'viewport';
  var popper = 'popper';
  var reference = 'reference';
  var variationPlacements = /*#__PURE__*/basePlacements.reduce(function (acc, placement) {
    return acc.concat([placement + "-" + start, placement + "-" + end]);
  }, []);
  var placements = /*#__PURE__*/[].concat(basePlacements, [auto]).reduce(function (acc, placement) {
    return acc.concat([placement, placement + "-" + start, placement + "-" + end]);
  }, []); // modifiers that need to read the DOM

  var beforeRead = 'beforeRead';
  var read = 'read';
  var afterRead = 'afterRead'; // pure-logic modifiers

  var beforeMain = 'beforeMain';
  var main = 'main';
  var afterMain = 'afterMain'; // modifier with the purpose to write to the DOM (or write into a framework state)

  var beforeWrite = 'beforeWrite';
  var write = 'write';
  var afterWrite = 'afterWrite';
  var modifierPhases = [beforeRead, read, afterRead, beforeMain, main, afterMain, beforeWrite, write, afterWrite];

  function order(modifiers) {
    var map = new Map();
    var visited = new Set();
    var result = [];
    modifiers.forEach(function (modifier) {
      map.set(modifier.name, modifier);
    }); // On visiting object, check for its dependencies and visit them recursively

    function sort(modifier) {
      visited.add(modifier.name);
      var requires = [].concat(modifier.requires || [], modifier.requiresIfExists || []);
      requires.forEach(function (dep) {
        if (!visited.has(dep)) {
          var depModifier = map.get(dep);

          if (depModifier) {
            sort(depModifier);
          }
        }
      });
      result.push(modifier);
    }

    modifiers.forEach(function (modifier) {
      if (!visited.has(modifier.name)) {
        // check for visited object
        sort(modifier);
      }
    });
    return result;
  }

  function orderModifiers(modifiers) {
    // order based on dependencies
    var orderedModifiers = order(modifiers); // order based on phase

    return modifierPhases.reduce(function (acc, phase) {
      return acc.concat(orderedModifiers.filter(function (modifier) {
        return modifier.phase === phase;
      }));
    }, []);
  }

  function debounce(fn) {
    var pending;
    return function () {
      if (!pending) {
        pending = new Promise(function (resolve) {
          Promise.resolve().then(function () {
            pending = undefined;
            resolve(fn());
          });
        });
      }

      return pending;
    };
  }

  function mergeByName(modifiers) {
    var merged = modifiers.reduce(function (merged, current) {
      var existing = merged[current.name];
      merged[current.name] = existing ? Object.assign({}, existing, current, {
        options: Object.assign({}, existing.options, current.options),
        data: Object.assign({}, existing.data, current.data)
      }) : current;
      return merged;
    }, {}); // IE11 does not support Object.values

    return Object.keys(merged).map(function (key) {
      return merged[key];
    });
  }

  function getViewportRect(element, strategy) {
    var win = getWindow(element);
    var html = getDocumentElement(element);
    var visualViewport = win.visualViewport;
    var width = html.clientWidth;
    var height = html.clientHeight;
    var x = 0;
    var y = 0;

    if (visualViewport) {
      width = visualViewport.width;
      height = visualViewport.height;
      var layoutViewport = isLayoutViewport();

      if (layoutViewport || !layoutViewport && strategy === 'fixed') {
        x = visualViewport.offsetLeft;
        y = visualViewport.offsetTop;
      }
    }

    return {
      width: width,
      height: height,
      x: x + getWindowScrollBarX(element),
      y: y
    };
  }

  // of the `<html>` and `<body>` rect bounds if horizontally scrollable

  function getDocumentRect(element) {
    var _element$ownerDocumen;

    var html = getDocumentElement(element);
    var winScroll = getWindowScroll(element);
    var body = (_element$ownerDocumen = element.ownerDocument) == null ? void 0 : _element$ownerDocumen.body;
    var width = max(html.scrollWidth, html.clientWidth, body ? body.scrollWidth : 0, body ? body.clientWidth : 0);
    var height = max(html.scrollHeight, html.clientHeight, body ? body.scrollHeight : 0, body ? body.clientHeight : 0);
    var x = -winScroll.scrollLeft + getWindowScrollBarX(element);
    var y = -winScroll.scrollTop;

    if (getComputedStyle(body || html).direction === 'rtl') {
      x += max(html.clientWidth, body ? body.clientWidth : 0) - width;
    }

    return {
      width: width,
      height: height,
      x: x,
      y: y
    };
  }

  function contains(parent, child) {
    var rootNode = child.getRootNode && child.getRootNode(); // First, attempt with faster native method

    if (parent.contains(child)) {
      return true;
    } // then fallback to custom implementation with Shadow DOM support
    else if (rootNode && isShadowRoot(rootNode)) {
        var next = child;

        do {
          if (next && parent.isSameNode(next)) {
            return true;
          } // $FlowFixMe[prop-missing]: need a better way to handle this...


          next = next.parentNode || next.host;
        } while (next);
      } // Give up, the result is false


    return false;
  }

  function rectToClientRect(rect) {
    return Object.assign({}, rect, {
      left: rect.x,
      top: rect.y,
      right: rect.x + rect.width,
      bottom: rect.y + rect.height
    });
  }

  function getInnerBoundingClientRect(element, strategy) {
    var rect = getBoundingClientRect(element, false, strategy === 'fixed');
    rect.top = rect.top + element.clientTop;
    rect.left = rect.left + element.clientLeft;
    rect.bottom = rect.top + element.clientHeight;
    rect.right = rect.left + element.clientWidth;
    rect.width = element.clientWidth;
    rect.height = element.clientHeight;
    rect.x = rect.left;
    rect.y = rect.top;
    return rect;
  }

  function getClientRectFromMixedType(element, clippingParent, strategy) {
    return clippingParent === viewport ? rectToClientRect(getViewportRect(element, strategy)) : isElement(clippingParent) ? getInnerBoundingClientRect(clippingParent, strategy) : rectToClientRect(getDocumentRect(getDocumentElement(element)));
  } // A "clipping parent" is an overflowable container with the characteristic of
  // clipping (or hiding) overflowing elements with a position different from
  // `initial`


  function getClippingParents(element) {
    var clippingParents = listScrollParents(getParentNode(element));
    var canEscapeClipping = ['absolute', 'fixed'].indexOf(getComputedStyle(element).position) >= 0;
    var clipperElement = canEscapeClipping && isHTMLElement(element) ? getOffsetParent(element) : element;

    if (!isElement(clipperElement)) {
      return [];
    } // $FlowFixMe[incompatible-return]: https://github.com/facebook/flow/issues/1414


    return clippingParents.filter(function (clippingParent) {
      return isElement(clippingParent) && contains(clippingParent, clipperElement) && getNodeName(clippingParent) !== 'body';
    });
  } // Gets the maximum area that the element is visible in due to any number of
  // clipping parents


  function getClippingRect(element, boundary, rootBoundary, strategy) {
    var mainClippingParents = boundary === 'clippingParents' ? getClippingParents(element) : [].concat(boundary);
    var clippingParents = [].concat(mainClippingParents, [rootBoundary]);
    var firstClippingParent = clippingParents[0];
    var clippingRect = clippingParents.reduce(function (accRect, clippingParent) {
      var rect = getClientRectFromMixedType(element, clippingParent, strategy);
      accRect.top = max(rect.top, accRect.top);
      accRect.right = min(rect.right, accRect.right);
      accRect.bottom = min(rect.bottom, accRect.bottom);
      accRect.left = max(rect.left, accRect.left);
      return accRect;
    }, getClientRectFromMixedType(element, firstClippingParent, strategy));
    clippingRect.width = clippingRect.right - clippingRect.left;
    clippingRect.height = clippingRect.bottom - clippingRect.top;
    clippingRect.x = clippingRect.left;
    clippingRect.y = clippingRect.top;
    return clippingRect;
  }

  function getBasePlacement(placement) {
    return placement.split('-')[0];
  }

  function getVariation(placement) {
    return placement.split('-')[1];
  }

  function getMainAxisFromPlacement(placement) {
    return ['top', 'bottom'].indexOf(placement) >= 0 ? 'x' : 'y';
  }

  function computeOffsets(_ref) {
    var reference = _ref.reference,
        element = _ref.element,
        placement = _ref.placement;
    var basePlacement = placement ? getBasePlacement(placement) : null;
    var variation = placement ? getVariation(placement) : null;
    var commonX = reference.x + reference.width / 2 - element.width / 2;
    var commonY = reference.y + reference.height / 2 - element.height / 2;
    var offsets;

    switch (basePlacement) {
      case top:
        offsets = {
          x: commonX,
          y: reference.y - element.height
        };
        break;

      case bottom:
        offsets = {
          x: commonX,
          y: reference.y + reference.height
        };
        break;

      case right:
        offsets = {
          x: reference.x + reference.width,
          y: commonY
        };
        break;

      case left:
        offsets = {
          x: reference.x - element.width,
          y: commonY
        };
        break;

      default:
        offsets = {
          x: reference.x,
          y: reference.y
        };
    }

    var mainAxis = basePlacement ? getMainAxisFromPlacement(basePlacement) : null;

    if (mainAxis != null) {
      var len = mainAxis === 'y' ? 'height' : 'width';

      switch (variation) {
        case start:
          offsets[mainAxis] = offsets[mainAxis] - (reference[len] / 2 - element[len] / 2);
          break;

        case end:
          offsets[mainAxis] = offsets[mainAxis] + (reference[len] / 2 - element[len] / 2);
          break;
      }
    }

    return offsets;
  }

  function getFreshSideObject() {
    return {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
    };
  }

  function mergePaddingObject(paddingObject) {
    return Object.assign({}, getFreshSideObject(), paddingObject);
  }

  function expandToHashMap(value, keys) {
    return keys.reduce(function (hashMap, key) {
      hashMap[key] = value;
      return hashMap;
    }, {});
  }

  function detectOverflow(state, options) {
    if (options === void 0) {
      options = {};
    }

    var _options = options,
        _options$placement = _options.placement,
        placement = _options$placement === void 0 ? state.placement : _options$placement,
        _options$strategy = _options.strategy,
        strategy = _options$strategy === void 0 ? state.strategy : _options$strategy,
        _options$boundary = _options.boundary,
        boundary = _options$boundary === void 0 ? clippingParents : _options$boundary,
        _options$rootBoundary = _options.rootBoundary,
        rootBoundary = _options$rootBoundary === void 0 ? viewport : _options$rootBoundary,
        _options$elementConte = _options.elementContext,
        elementContext = _options$elementConte === void 0 ? popper : _options$elementConte,
        _options$altBoundary = _options.altBoundary,
        altBoundary = _options$altBoundary === void 0 ? false : _options$altBoundary,
        _options$padding = _options.padding,
        padding = _options$padding === void 0 ? 0 : _options$padding;
    var paddingObject = mergePaddingObject(typeof padding !== 'number' ? padding : expandToHashMap(padding, basePlacements));
    var altContext = elementContext === popper ? reference : popper;
    var popperRect = state.rects.popper;
    var element = state.elements[altBoundary ? altContext : elementContext];
    var clippingClientRect = getClippingRect(isElement(element) ? element : element.contextElement || getDocumentElement(state.elements.popper), boundary, rootBoundary, strategy);
    var referenceClientRect = getBoundingClientRect(state.elements.reference);
    var popperOffsets = computeOffsets({
      reference: referenceClientRect,
      element: popperRect,
      strategy: 'absolute',
      placement: placement
    });
    var popperClientRect = rectToClientRect(Object.assign({}, popperRect, popperOffsets));
    var elementClientRect = elementContext === popper ? popperClientRect : referenceClientRect; // positive = overflowing the clipping rect
    // 0 or negative = within the clipping rect

    var overflowOffsets = {
      top: clippingClientRect.top - elementClientRect.top + paddingObject.top,
      bottom: elementClientRect.bottom - clippingClientRect.bottom + paddingObject.bottom,
      left: clippingClientRect.left - elementClientRect.left + paddingObject.left,
      right: elementClientRect.right - clippingClientRect.right + paddingObject.right
    };
    var offsetData = state.modifiersData.offset; // Offsets can be applied only to the popper element

    if (elementContext === popper && offsetData) {
      var offset = offsetData[placement];
      Object.keys(overflowOffsets).forEach(function (key) {
        var multiply = [right, bottom].indexOf(key) >= 0 ? 1 : -1;
        var axis = [top, bottom].indexOf(key) >= 0 ? 'y' : 'x';
        overflowOffsets[key] += offset[axis] * multiply;
      });
    }

    return overflowOffsets;
  }

  var DEFAULT_OPTIONS = {
    placement: 'bottom',
    modifiers: [],
    strategy: 'absolute'
  };

  function areValidElements() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return !args.some(function (element) {
      return !(element && typeof element.getBoundingClientRect === 'function');
    });
  }

  function popperGenerator(generatorOptions) {
    if (generatorOptions === void 0) {
      generatorOptions = {};
    }

    var _generatorOptions = generatorOptions,
        _generatorOptions$def = _generatorOptions.defaultModifiers,
        defaultModifiers = _generatorOptions$def === void 0 ? [] : _generatorOptions$def,
        _generatorOptions$def2 = _generatorOptions.defaultOptions,
        defaultOptions = _generatorOptions$def2 === void 0 ? DEFAULT_OPTIONS : _generatorOptions$def2;
    return function createPopper(reference, popper, options) {
      if (options === void 0) {
        options = defaultOptions;
      }

      var state = {
        placement: 'bottom',
        orderedModifiers: [],
        options: Object.assign({}, DEFAULT_OPTIONS, defaultOptions),
        modifiersData: {},
        elements: {
          reference: reference,
          popper: popper
        },
        attributes: {},
        styles: {}
      };
      var effectCleanupFns = [];
      var isDestroyed = false;
      var instance = {
        state: state,
        setOptions: function setOptions(setOptionsAction) {
          var options = typeof setOptionsAction === 'function' ? setOptionsAction(state.options) : setOptionsAction;
          cleanupModifierEffects();
          state.options = Object.assign({}, defaultOptions, state.options, options);
          state.scrollParents = {
            reference: isElement(reference) ? listScrollParents(reference) : reference.contextElement ? listScrollParents(reference.contextElement) : [],
            popper: listScrollParents(popper)
          }; // Orders the modifiers based on their dependencies and `phase`
          // properties

          var orderedModifiers = orderModifiers(mergeByName([].concat(defaultModifiers, state.options.modifiers))); // Strip out disabled modifiers

          state.orderedModifiers = orderedModifiers.filter(function (m) {
            return m.enabled;
          });
          runModifierEffects();
          return instance.update();
        },
        // Sync update – it will always be executed, even if not necessary. This
        // is useful for low frequency updates where sync behavior simplifies the
        // logic.
        // For high frequency updates (e.g. `resize` and `scroll` events), always
        // prefer the async Popper#update method
        forceUpdate: function forceUpdate() {
          if (isDestroyed) {
            return;
          }

          var _state$elements = state.elements,
              reference = _state$elements.reference,
              popper = _state$elements.popper; // Don't proceed if `reference` or `popper` are not valid elements
          // anymore

          if (!areValidElements(reference, popper)) {
            return;
          } // Store the reference and popper rects to be read by modifiers


          state.rects = {
            reference: getCompositeRect(reference, getOffsetParent(popper), state.options.strategy === 'fixed'),
            popper: getLayoutRect(popper)
          }; // Modifiers have the ability to reset the current update cycle. The
          // most common use case for this is the `flip` modifier changing the
          // placement, which then needs to re-run all the modifiers, because the
          // logic was previously ran for the previous placement and is therefore
          // stale/incorrect

          state.reset = false;
          state.placement = state.options.placement; // On each update cycle, the `modifiersData` property for each modifier
          // is filled with the initial data specified by the modifier. This means
          // it doesn't persist and is fresh on each update.
          // To ensure persistent data, use `${name}#persistent`

          state.orderedModifiers.forEach(function (modifier) {
            return state.modifiersData[modifier.name] = Object.assign({}, modifier.data);
          });

          for (var index = 0; index < state.orderedModifiers.length; index++) {
            if (state.reset === true) {
              state.reset = false;
              index = -1;
              continue;
            }

            var _state$orderedModifie = state.orderedModifiers[index],
                fn = _state$orderedModifie.fn,
                _state$orderedModifie2 = _state$orderedModifie.options,
                _options = _state$orderedModifie2 === void 0 ? {} : _state$orderedModifie2,
                name = _state$orderedModifie.name;

            if (typeof fn === 'function') {
              state = fn({
                state: state,
                options: _options,
                name: name,
                instance: instance
              }) || state;
            }
          }
        },
        // Async and optimistically optimized update – it will not be executed if
        // not necessary (debounced to run at most once-per-tick)
        update: debounce(function () {
          return new Promise(function (resolve) {
            instance.forceUpdate();
            resolve(state);
          });
        }),
        destroy: function destroy() {
          cleanupModifierEffects();
          isDestroyed = true;
        }
      };

      if (!areValidElements(reference, popper)) {
        return instance;
      }

      instance.setOptions(options).then(function (state) {
        if (!isDestroyed && options.onFirstUpdate) {
          options.onFirstUpdate(state);
        }
      }); // Modifiers have the ability to execute arbitrary code before the first
      // update cycle runs. They will be executed in the same order as the update
      // cycle. This is useful when a modifier adds some persistent data that
      // other modifiers need to use, but the modifier is run after the dependent
      // one.

      function runModifierEffects() {
        state.orderedModifiers.forEach(function (_ref) {
          var name = _ref.name,
              _ref$options = _ref.options,
              options = _ref$options === void 0 ? {} : _ref$options,
              effect = _ref.effect;

          if (typeof effect === 'function') {
            var cleanupFn = effect({
              state: state,
              name: name,
              instance: instance,
              options: options
            });

            var noopFn = function noopFn() {};

            effectCleanupFns.push(cleanupFn || noopFn);
          }
        });
      }

      function cleanupModifierEffects() {
        effectCleanupFns.forEach(function (fn) {
          return fn();
        });
        effectCleanupFns = [];
      }

      return instance;
    };
  }

  var passive = {
    passive: true
  };

  function effect$1(_ref) {
    var state = _ref.state,
        instance = _ref.instance,
        options = _ref.options;
    var _options$scroll = options.scroll,
        scroll = _options$scroll === void 0 ? true : _options$scroll,
        _options$resize = options.resize,
        resize = _options$resize === void 0 ? true : _options$resize;
    var window = getWindow(state.elements.popper);
    var scrollParents = [].concat(state.scrollParents.reference, state.scrollParents.popper);

    if (scroll) {
      scrollParents.forEach(function (scrollParent) {
        scrollParent.addEventListener('scroll', instance.update, passive);
      });
    }

    if (resize) {
      window.addEventListener('resize', instance.update, passive);
    }

    return function () {
      if (scroll) {
        scrollParents.forEach(function (scrollParent) {
          scrollParent.removeEventListener('scroll', instance.update, passive);
        });
      }

      if (resize) {
        window.removeEventListener('resize', instance.update, passive);
      }
    };
  } // eslint-disable-next-line import/no-unused-modules


  var eventListeners = {
    name: 'eventListeners',
    enabled: true,
    phase: 'write',
    fn: function fn() {},
    effect: effect$1,
    data: {}
  };

  function popperOffsets(_ref) {
    var state = _ref.state,
        name = _ref.name;
    // Offsets are the actual position the popper needs to have to be
    // properly positioned near its reference element
    // This is the most basic placement, and will be adjusted by
    // the modifiers in the next step
    state.modifiersData[name] = computeOffsets({
      reference: state.rects.reference,
      element: state.rects.popper,
      strategy: 'absolute',
      placement: state.placement
    });
  } // eslint-disable-next-line import/no-unused-modules


  var popperOffsets$1 = {
    name: 'popperOffsets',
    enabled: true,
    phase: 'read',
    fn: popperOffsets,
    data: {}
  };

  var unsetSides = {
    top: 'auto',
    right: 'auto',
    bottom: 'auto',
    left: 'auto'
  }; // Round the offsets to the nearest suitable subpixel based on the DPR.
  // Zooming can change the DPR, but it seems to report a value that will
  // cleanly divide the values into the appropriate subpixels.

  function roundOffsetsByDPR(_ref, win) {
    var x = _ref.x,
        y = _ref.y;
    var dpr = win.devicePixelRatio || 1;
    return {
      x: round(x * dpr) / dpr || 0,
      y: round(y * dpr) / dpr || 0
    };
  }

  function mapToStyles(_ref2) {
    var _Object$assign2;

    var popper = _ref2.popper,
        popperRect = _ref2.popperRect,
        placement = _ref2.placement,
        variation = _ref2.variation,
        offsets = _ref2.offsets,
        position = _ref2.position,
        gpuAcceleration = _ref2.gpuAcceleration,
        adaptive = _ref2.adaptive,
        roundOffsets = _ref2.roundOffsets,
        isFixed = _ref2.isFixed;
    var _offsets$x = offsets.x,
        x = _offsets$x === void 0 ? 0 : _offsets$x,
        _offsets$y = offsets.y,
        y = _offsets$y === void 0 ? 0 : _offsets$y;

    var _ref3 = typeof roundOffsets === 'function' ? roundOffsets({
      x: x,
      y: y
    }) : {
      x: x,
      y: y
    };

    x = _ref3.x;
    y = _ref3.y;
    var hasX = offsets.hasOwnProperty('x');
    var hasY = offsets.hasOwnProperty('y');
    var sideX = left;
    var sideY = top;
    var win = window;

    if (adaptive) {
      var offsetParent = getOffsetParent(popper);
      var heightProp = 'clientHeight';
      var widthProp = 'clientWidth';

      if (offsetParent === getWindow(popper)) {
        offsetParent = getDocumentElement(popper);

        if (getComputedStyle(offsetParent).position !== 'static' && position === 'absolute') {
          heightProp = 'scrollHeight';
          widthProp = 'scrollWidth';
        }
      } // $FlowFixMe[incompatible-cast]: force type refinement, we compare offsetParent with window above, but Flow doesn't detect it


      offsetParent = offsetParent;

      if (placement === top || (placement === left || placement === right) && variation === end) {
        sideY = bottom;
        var offsetY = isFixed && offsetParent === win && win.visualViewport ? win.visualViewport.height : // $FlowFixMe[prop-missing]
        offsetParent[heightProp];
        y -= offsetY - popperRect.height;
        y *= gpuAcceleration ? 1 : -1;
      }

      if (placement === left || (placement === top || placement === bottom) && variation === end) {
        sideX = right;
        var offsetX = isFixed && offsetParent === win && win.visualViewport ? win.visualViewport.width : // $FlowFixMe[prop-missing]
        offsetParent[widthProp];
        x -= offsetX - popperRect.width;
        x *= gpuAcceleration ? 1 : -1;
      }
    }

    var commonStyles = Object.assign({
      position: position
    }, adaptive && unsetSides);

    var _ref4 = roundOffsets === true ? roundOffsetsByDPR({
      x: x,
      y: y
    }, getWindow(popper)) : {
      x: x,
      y: y
    };

    x = _ref4.x;
    y = _ref4.y;

    if (gpuAcceleration) {
      var _Object$assign;

      return Object.assign({}, commonStyles, (_Object$assign = {}, _Object$assign[sideY] = hasY ? '0' : '', _Object$assign[sideX] = hasX ? '0' : '', _Object$assign.transform = (win.devicePixelRatio || 1) <= 1 ? "translate(" + x + "px, " + y + "px)" : "translate3d(" + x + "px, " + y + "px, 0)", _Object$assign));
    }

    return Object.assign({}, commonStyles, (_Object$assign2 = {}, _Object$assign2[sideY] = hasY ? y + "px" : '', _Object$assign2[sideX] = hasX ? x + "px" : '', _Object$assign2.transform = '', _Object$assign2));
  }

  function computeStyles(_ref5) {
    var state = _ref5.state,
        options = _ref5.options;
    var _options$gpuAccelerat = options.gpuAcceleration,
        gpuAcceleration = _options$gpuAccelerat === void 0 ? true : _options$gpuAccelerat,
        _options$adaptive = options.adaptive,
        adaptive = _options$adaptive === void 0 ? true : _options$adaptive,
        _options$roundOffsets = options.roundOffsets,
        roundOffsets = _options$roundOffsets === void 0 ? true : _options$roundOffsets;
    var commonStyles = {
      placement: getBasePlacement(state.placement),
      variation: getVariation(state.placement),
      popper: state.elements.popper,
      popperRect: state.rects.popper,
      gpuAcceleration: gpuAcceleration,
      isFixed: state.options.strategy === 'fixed'
    };

    if (state.modifiersData.popperOffsets != null) {
      state.styles.popper = Object.assign({}, state.styles.popper, mapToStyles(Object.assign({}, commonStyles, {
        offsets: state.modifiersData.popperOffsets,
        position: state.options.strategy,
        adaptive: adaptive,
        roundOffsets: roundOffsets
      })));
    }

    if (state.modifiersData.arrow != null) {
      state.styles.arrow = Object.assign({}, state.styles.arrow, mapToStyles(Object.assign({}, commonStyles, {
        offsets: state.modifiersData.arrow,
        position: 'absolute',
        adaptive: false,
        roundOffsets: roundOffsets
      })));
    }

    state.attributes.popper = Object.assign({}, state.attributes.popper, {
      'data-popper-placement': state.placement
    });
  } // eslint-disable-next-line import/no-unused-modules


  var computeStyles$1 = {
    name: 'computeStyles',
    enabled: true,
    phase: 'beforeWrite',
    fn: computeStyles,
    data: {}
  };

  // and applies them to the HTMLElements such as popper and arrow

  function applyStyles(_ref) {
    var state = _ref.state;
    Object.keys(state.elements).forEach(function (name) {
      var style = state.styles[name] || {};
      var attributes = state.attributes[name] || {};
      var element = state.elements[name]; // arrow is optional + virtual elements

      if (!isHTMLElement(element) || !getNodeName(element)) {
        return;
      } // Flow doesn't support to extend this property, but it's the most
      // effective way to apply styles to an HTMLElement
      // $FlowFixMe[cannot-write]


      Object.assign(element.style, style);
      Object.keys(attributes).forEach(function (name) {
        var value = attributes[name];

        if (value === false) {
          element.removeAttribute(name);
        } else {
          element.setAttribute(name, value === true ? '' : value);
        }
      });
    });
  }

  function effect(_ref2) {
    var state = _ref2.state;
    var initialStyles = {
      popper: {
        position: state.options.strategy,
        left: '0',
        top: '0',
        margin: '0'
      },
      arrow: {
        position: 'absolute'
      },
      reference: {}
    };
    Object.assign(state.elements.popper.style, initialStyles.popper);
    state.styles = initialStyles;

    if (state.elements.arrow) {
      Object.assign(state.elements.arrow.style, initialStyles.arrow);
    }

    return function () {
      Object.keys(state.elements).forEach(function (name) {
        var element = state.elements[name];
        var attributes = state.attributes[name] || {};
        var styleProperties = Object.keys(state.styles.hasOwnProperty(name) ? state.styles[name] : initialStyles[name]); // Set all values to an empty string to unset them

        var style = styleProperties.reduce(function (style, property) {
          style[property] = '';
          return style;
        }, {}); // arrow is optional + virtual elements

        if (!isHTMLElement(element) || !getNodeName(element)) {
          return;
        }

        Object.assign(element.style, style);
        Object.keys(attributes).forEach(function (attribute) {
          element.removeAttribute(attribute);
        });
      });
    };
  } // eslint-disable-next-line import/no-unused-modules


  var applyStyles$1 = {
    name: 'applyStyles',
    enabled: true,
    phase: 'write',
    fn: applyStyles,
    effect: effect,
    requires: ['computeStyles']
  };

  var defaultModifiers = [eventListeners, popperOffsets$1, computeStyles$1, applyStyles$1];
  var createPopper$1 = /*#__PURE__*/popperGenerator({
    defaultModifiers: defaultModifiers
  }); // eslint-disable-next-line import/no-unused-modules

  var hash$1 = {
    left: 'right',
    right: 'left',
    bottom: 'top',
    top: 'bottom'
  };
  function getOppositePlacement(placement) {
    return placement.replace(/left|right|bottom|top/g, function (matched) {
      return hash$1[matched];
    });
  }

  var hash = {
    start: 'end',
    end: 'start'
  };
  function getOppositeVariationPlacement(placement) {
    return placement.replace(/start|end/g, function (matched) {
      return hash[matched];
    });
  }

  function computeAutoPlacement(state, options) {
    if (options === void 0) {
      options = {};
    }

    var _options = options,
        placement = _options.placement,
        boundary = _options.boundary,
        rootBoundary = _options.rootBoundary,
        padding = _options.padding,
        flipVariations = _options.flipVariations,
        _options$allowedAutoP = _options.allowedAutoPlacements,
        allowedAutoPlacements = _options$allowedAutoP === void 0 ? placements : _options$allowedAutoP;
    var variation = getVariation(placement);
    var placements$1 = variation ? flipVariations ? variationPlacements : variationPlacements.filter(function (placement) {
      return getVariation(placement) === variation;
    }) : basePlacements;
    var allowedPlacements = placements$1.filter(function (placement) {
      return allowedAutoPlacements.indexOf(placement) >= 0;
    });

    if (allowedPlacements.length === 0) {
      allowedPlacements = placements$1;
    } // $FlowFixMe[incompatible-type]: Flow seems to have problems with two array unions...


    var overflows = allowedPlacements.reduce(function (acc, placement) {
      acc[placement] = detectOverflow(state, {
        placement: placement,
        boundary: boundary,
        rootBoundary: rootBoundary,
        padding: padding
      })[getBasePlacement(placement)];
      return acc;
    }, {});
    return Object.keys(overflows).sort(function (a, b) {
      return overflows[a] - overflows[b];
    });
  }

  function getExpandedFallbackPlacements(placement) {
    if (getBasePlacement(placement) === auto) {
      return [];
    }

    var oppositePlacement = getOppositePlacement(placement);
    return [getOppositeVariationPlacement(placement), oppositePlacement, getOppositeVariationPlacement(oppositePlacement)];
  }

  function flip(_ref) {
    var state = _ref.state,
        options = _ref.options,
        name = _ref.name;

    if (state.modifiersData[name]._skip) {
      return;
    }

    var _options$mainAxis = options.mainAxis,
        checkMainAxis = _options$mainAxis === void 0 ? true : _options$mainAxis,
        _options$altAxis = options.altAxis,
        checkAltAxis = _options$altAxis === void 0 ? true : _options$altAxis,
        specifiedFallbackPlacements = options.fallbackPlacements,
        padding = options.padding,
        boundary = options.boundary,
        rootBoundary = options.rootBoundary,
        altBoundary = options.altBoundary,
        _options$flipVariatio = options.flipVariations,
        flipVariations = _options$flipVariatio === void 0 ? true : _options$flipVariatio,
        allowedAutoPlacements = options.allowedAutoPlacements;
    var preferredPlacement = state.options.placement;
    var basePlacement = getBasePlacement(preferredPlacement);
    var isBasePlacement = basePlacement === preferredPlacement;
    var fallbackPlacements = specifiedFallbackPlacements || (isBasePlacement || !flipVariations ? [getOppositePlacement(preferredPlacement)] : getExpandedFallbackPlacements(preferredPlacement));
    var placements = [preferredPlacement].concat(fallbackPlacements).reduce(function (acc, placement) {
      return acc.concat(getBasePlacement(placement) === auto ? computeAutoPlacement(state, {
        placement: placement,
        boundary: boundary,
        rootBoundary: rootBoundary,
        padding: padding,
        flipVariations: flipVariations,
        allowedAutoPlacements: allowedAutoPlacements
      }) : placement);
    }, []);
    var referenceRect = state.rects.reference;
    var popperRect = state.rects.popper;
    var checksMap = new Map();
    var makeFallbackChecks = true;
    var firstFittingPlacement = placements[0];

    for (var i = 0; i < placements.length; i++) {
      var placement = placements[i];

      var _basePlacement = getBasePlacement(placement);

      var isStartVariation = getVariation(placement) === start;
      var isVertical = [top, bottom].indexOf(_basePlacement) >= 0;
      var len = isVertical ? 'width' : 'height';
      var overflow = detectOverflow(state, {
        placement: placement,
        boundary: boundary,
        rootBoundary: rootBoundary,
        altBoundary: altBoundary,
        padding: padding
      });
      var mainVariationSide = isVertical ? isStartVariation ? right : left : isStartVariation ? bottom : top;

      if (referenceRect[len] > popperRect[len]) {
        mainVariationSide = getOppositePlacement(mainVariationSide);
      }

      var altVariationSide = getOppositePlacement(mainVariationSide);
      var checks = [];

      if (checkMainAxis) {
        checks.push(overflow[_basePlacement] <= 0);
      }

      if (checkAltAxis) {
        checks.push(overflow[mainVariationSide] <= 0, overflow[altVariationSide] <= 0);
      }

      if (checks.every(function (check) {
        return check;
      })) {
        firstFittingPlacement = placement;
        makeFallbackChecks = false;
        break;
      }

      checksMap.set(placement, checks);
    }

    if (makeFallbackChecks) {
      // `2` may be desired in some cases – research later
      var numberOfChecks = flipVariations ? 3 : 1;

      var _loop = function _loop(_i) {
        var fittingPlacement = placements.find(function (placement) {
          var checks = checksMap.get(placement);

          if (checks) {
            return checks.slice(0, _i).every(function (check) {
              return check;
            });
          }
        });

        if (fittingPlacement) {
          firstFittingPlacement = fittingPlacement;
          return "break";
        }
      };

      for (var _i = numberOfChecks; _i > 0; _i--) {
        var _ret = _loop(_i);

        if (_ret === "break") break;
      }
    }

    if (state.placement !== firstFittingPlacement) {
      state.modifiersData[name]._skip = true;
      state.placement = firstFittingPlacement;
      state.reset = true;
    }
  } // eslint-disable-next-line import/no-unused-modules


  var flip$1 = {
    name: 'flip',
    enabled: true,
    phase: 'main',
    fn: flip,
    requiresIfExists: ['offset'],
    data: {
      _skip: false
    }
  };

  function getAltAxis(axis) {
    return axis === 'x' ? 'y' : 'x';
  }

  function within(min$1, value, max$1) {
    return max(min$1, min(value, max$1));
  }
  function withinMaxClamp(min, value, max) {
    var v = within(min, value, max);
    return v > max ? max : v;
  }

  function preventOverflow(_ref) {
    var state = _ref.state,
        options = _ref.options,
        name = _ref.name;
    var _options$mainAxis = options.mainAxis,
        checkMainAxis = _options$mainAxis === void 0 ? true : _options$mainAxis,
        _options$altAxis = options.altAxis,
        checkAltAxis = _options$altAxis === void 0 ? false : _options$altAxis,
        boundary = options.boundary,
        rootBoundary = options.rootBoundary,
        altBoundary = options.altBoundary,
        padding = options.padding,
        _options$tether = options.tether,
        tether = _options$tether === void 0 ? true : _options$tether,
        _options$tetherOffset = options.tetherOffset,
        tetherOffset = _options$tetherOffset === void 0 ? 0 : _options$tetherOffset;
    var overflow = detectOverflow(state, {
      boundary: boundary,
      rootBoundary: rootBoundary,
      padding: padding,
      altBoundary: altBoundary
    });
    var basePlacement = getBasePlacement(state.placement);
    var variation = getVariation(state.placement);
    var isBasePlacement = !variation;
    var mainAxis = getMainAxisFromPlacement(basePlacement);
    var altAxis = getAltAxis(mainAxis);
    var popperOffsets = state.modifiersData.popperOffsets;
    var referenceRect = state.rects.reference;
    var popperRect = state.rects.popper;
    var tetherOffsetValue = typeof tetherOffset === 'function' ? tetherOffset(Object.assign({}, state.rects, {
      placement: state.placement
    })) : tetherOffset;
    var normalizedTetherOffsetValue = typeof tetherOffsetValue === 'number' ? {
      mainAxis: tetherOffsetValue,
      altAxis: tetherOffsetValue
    } : Object.assign({
      mainAxis: 0,
      altAxis: 0
    }, tetherOffsetValue);
    var offsetModifierState = state.modifiersData.offset ? state.modifiersData.offset[state.placement] : null;
    var data = {
      x: 0,
      y: 0
    };

    if (!popperOffsets) {
      return;
    }

    if (checkMainAxis) {
      var _offsetModifierState$;

      var mainSide = mainAxis === 'y' ? top : left;
      var altSide = mainAxis === 'y' ? bottom : right;
      var len = mainAxis === 'y' ? 'height' : 'width';
      var offset = popperOffsets[mainAxis];
      var min$1 = offset + overflow[mainSide];
      var max$1 = offset - overflow[altSide];
      var additive = tether ? -popperRect[len] / 2 : 0;
      var minLen = variation === start ? referenceRect[len] : popperRect[len];
      var maxLen = variation === start ? -popperRect[len] : -referenceRect[len]; // We need to include the arrow in the calculation so the arrow doesn't go
      // outside the reference bounds

      var arrowElement = state.elements.arrow;
      var arrowRect = tether && arrowElement ? getLayoutRect(arrowElement) : {
        width: 0,
        height: 0
      };
      var arrowPaddingObject = state.modifiersData['arrow#persistent'] ? state.modifiersData['arrow#persistent'].padding : getFreshSideObject();
      var arrowPaddingMin = arrowPaddingObject[mainSide];
      var arrowPaddingMax = arrowPaddingObject[altSide]; // If the reference length is smaller than the arrow length, we don't want
      // to include its full size in the calculation. If the reference is small
      // and near the edge of a boundary, the popper can overflow even if the
      // reference is not overflowing as well (e.g. virtual elements with no
      // width or height)

      var arrowLen = within(0, referenceRect[len], arrowRect[len]);
      var minOffset = isBasePlacement ? referenceRect[len] / 2 - additive - arrowLen - arrowPaddingMin - normalizedTetherOffsetValue.mainAxis : minLen - arrowLen - arrowPaddingMin - normalizedTetherOffsetValue.mainAxis;
      var maxOffset = isBasePlacement ? -referenceRect[len] / 2 + additive + arrowLen + arrowPaddingMax + normalizedTetherOffsetValue.mainAxis : maxLen + arrowLen + arrowPaddingMax + normalizedTetherOffsetValue.mainAxis;
      var arrowOffsetParent = state.elements.arrow && getOffsetParent(state.elements.arrow);
      var clientOffset = arrowOffsetParent ? mainAxis === 'y' ? arrowOffsetParent.clientTop || 0 : arrowOffsetParent.clientLeft || 0 : 0;
      var offsetModifierValue = (_offsetModifierState$ = offsetModifierState == null ? void 0 : offsetModifierState[mainAxis]) != null ? _offsetModifierState$ : 0;
      var tetherMin = offset + minOffset - offsetModifierValue - clientOffset;
      var tetherMax = offset + maxOffset - offsetModifierValue;
      var preventedOffset = within(tether ? min(min$1, tetherMin) : min$1, offset, tether ? max(max$1, tetherMax) : max$1);
      popperOffsets[mainAxis] = preventedOffset;
      data[mainAxis] = preventedOffset - offset;
    }

    if (checkAltAxis) {
      var _offsetModifierState$2;

      var _mainSide = mainAxis === 'x' ? top : left;

      var _altSide = mainAxis === 'x' ? bottom : right;

      var _offset = popperOffsets[altAxis];

      var _len = altAxis === 'y' ? 'height' : 'width';

      var _min = _offset + overflow[_mainSide];

      var _max = _offset - overflow[_altSide];

      var isOriginSide = [top, left].indexOf(basePlacement) !== -1;

      var _offsetModifierValue = (_offsetModifierState$2 = offsetModifierState == null ? void 0 : offsetModifierState[altAxis]) != null ? _offsetModifierState$2 : 0;

      var _tetherMin = isOriginSide ? _min : _offset - referenceRect[_len] - popperRect[_len] - _offsetModifierValue + normalizedTetherOffsetValue.altAxis;

      var _tetherMax = isOriginSide ? _offset + referenceRect[_len] + popperRect[_len] - _offsetModifierValue - normalizedTetherOffsetValue.altAxis : _max;

      var _preventedOffset = tether && isOriginSide ? withinMaxClamp(_tetherMin, _offset, _tetherMax) : within(tether ? _tetherMin : _min, _offset, tether ? _tetherMax : _max);

      popperOffsets[altAxis] = _preventedOffset;
      data[altAxis] = _preventedOffset - _offset;
    }

    state.modifiersData[name] = data;
  } // eslint-disable-next-line import/no-unused-modules


  var preventOverflow$1 = {
    name: 'preventOverflow',
    enabled: true,
    phase: 'main',
    fn: preventOverflow,
    requiresIfExists: ['offset']
  };

  preventOverflow$1.options = {
    padding: 8
  };
  function createPopper (ref, popper) {
    return createPopper$1(ref, popper, {
      placement: 'bottom',
      strategy: 'fixed',
      modifiers: [flip$1, preventOverflow$1]
    });
  }

  /**
   *
   * @param {Object} btnEl
   * @param {Number} margin
   */
  function getPopperMaxHeight (btnEl, margin) {
    var _btnEl$getBoundingCli = btnEl.getBoundingClientRect(),
      top = _btnEl$getBoundingCli.top,
      bottom = _btnEl$getBoundingCli.bottom;
    bottom = Math.max(document.documentElement.clientHeight || 0, window.innerHeight || 0) - bottom;
    return Math.max(top, bottom) - margin;
  }

  function getClassName(ins) {
    var _ins$optionsManager$s = ins.optionsManager.setting,
      showMorePopperClass = _ins$optionsManager$s.showMorePopperClass,
      tablistViewClass = _ins$optionsManager$s.tablistViewClass,
      verticalClass = _ins$optionsManager$s.verticalClass,
      rtlClass = _ins$optionsManager$s.rtlClass;
    return tablistViewClass + ' ' + verticalClass + ' ' + (ins.getOption('direction') == 'rtl' ? rtlClass + ' ' : '') + showMorePopperClass;
  }

  function Popper$1(getDeps, props) {
    var _getDeps = getDeps(),
      createPopper = _getDeps.createPopper,
      getPopperMaxHeight = _getDeps.getPopperMaxHeight,
      getClassName = _getDeps.getClassName,
      clk = _getDeps.clk;
    var TabsComponent = props.TabsComponent,
      instance = props.instance,
      hiddenTabIDs = props.hiddenTabIDs,
      btnRef = props.btnRef;
    var popperRef = React.useRef();
    var ref = React.useRef();
    ref.current = ref.current || {};
    React.useLayoutEffect(function () {
      popperRef.current.style.maxHeight = getPopperMaxHeight(btnRef.current, 15) + 'px';
      var popperIns = createPopper(btnRef.current, popperRef.current);
      ref.current.popperIns = popperIns;
      return function () {
        popperIns.destroy();
      };
    }, []);
    React.useLayoutEffect(function () {
      ref.current && ref.current.popperIns && ref.current.popperIns.update();
    }, [hiddenTabIDs]);
    var _instance$getData = instance.getData(),
      selectedTabID = _instance$getData.selectedTabID;
    var openedTabIDs = hiddenTabIDs ? hiddenTabIDs.split(',') : [];
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
      onClick: clk,
      ref: popperRef,
      className: getClassName(instance)
    }, /*#__PURE__*/React.createElement(TabsComponent, {
      selectedTabID: selectedTabID,
      openTabIDs: openedTabIDs,
      dir: instance.getOption('direction'),
      isVertical: true
    })));
  }

  var clk = function clk(e) {
    e.nativeEvent.stopImmediatePropagation();
  };
  var getDeps = function getDeps() {
    return {
      createPopper: createPopper,
      getPopperMaxHeight: getPopperMaxHeight,
      getClassName: getClassName,
      clk: clk
    };
  };
  var Popper = Popper$1.bind(null, getDeps);

  function _extends$3() { return _extends$3 = Object.assign ? Object.assign.bind() : function (n) { for (var e = 1; e < arguments.length; e++) { var t = arguments[e]; for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]); } return n; }, _extends$3.apply(null, arguments); }
  function Button(getDeps, props) {
    var _getDeps = getDeps(),
      Popper = _getDeps.Popper,
      Api = _getDeps.Api,
      ctx = _getDeps.ctx,
      components = _getDeps.components;
    var _useState = React.useState(false),
      open = _useState[0],
      setOpen = _useState[1];
    var btnRef = React.useRef();
    var ref = React.useRef();
    components.useForceUpdate();
    ref.current = ref.current || Api.call(ctx, components, setOpen);
    var onClick = React.useCallback(function (ev) {
      ref.current.onButtonClick(ev, open);
    }, [open]);
    React.useEffect(function () {
      ref.current.onMount();
      return function () {
        (ref == null ? void 0 : ref.current) && ref.current.onDestroy && ref.current.onDestroy();
      };
    }, []);
    var IconComponent = ctx.getOption('moreButtonPlugin_iconComponent');
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", ref.current.btnPropsGenerator(onClick, btnRef, open), /*#__PURE__*/React.createElement(IconComponent, {
      instance: props.instance
    })), open ? /*#__PURE__*/React.createElement(Popper, _extends$3({}, props, {
      instance: ctx,
      TabsComponent: ref.current.TabsComponent,
      btnRef: btnRef
    })) : null);
  }

  var _excluded = ["aria-controls", "aria-expanded", "aria-selected", "id"];
  function _extends$2() { return _extends$2 = Object.assign ? Object.assign.bind() : function (n) { for (var e = 1; e < arguments.length; e++) { var t = arguments[e]; for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]); } return n; }, _extends$2.apply(null, arguments); }
  function _objectWithoutPropertiesLoose(r, e) { if (null == r) return {}; var t = {}; for (var n in r) if ({}.hasOwnProperty.call(r, n)) { if (e.indexOf(n) >= 0) continue; t[n] = r[n]; } return t; }
  var _counter = 0;
  function Api(components, setOpen) {
    return new (this.helper.module(function (api) {
      this._components = components;
      this._api = api;
      this._counter = _counter++;
      this._buttonID = this._getButtonID();
      this._tablistID = this._getTabListID();
      this.TabsComponent = this._getTabsComponent();
      this._closePopper = function () {
        setOpen(false);
      }, this._togglePopper = function (open) {
        setOpen(!open);
      };
      this._onBackdropClick = this._onBackdropClick.bind(this);
      this._onSelectTab = this._onSelectTab.bind(this);
    }, {
      _getTabListID: function _getTabListID() {
        return "rc-dyn-tabs-popupcontainer_" + this._counter;
      },
      _getButtonID: function _getButtonID() {
        return "rc-dyn-tabs-more-button-" + this._counter;
      },
      btnPropsGenerator: function btnPropsGenerator(onClick, ref, isOpen) {
        var result = {
          onClick: onClick,
          ref: ref,
          className: this._api.getSetting('titleClass') + ' ' + this._api.getSetting('showMoreButtonClass'),
          title: this._api.getOption('moreButtonPlugin_buttonTooltip')
        };
        if (this._api.getOption('accessibility')) {
          result.tabIndex = 0;
          result.role = 'button';
          result['aria-haspopup'] = 'true';
          result['aria-label'] = result.title; //todo
          result.id = this._buttonID;
          result['aria-controls'] = this._tablistID;
          result['aria-expanded'] = isOpen;
        }
        return result;
      },
      _getTabsComponent: function _getTabsComponent() {
        var _this = this;
        var buttonID = this._buttonID;
        var tablistID = this._tablistID;
        var _TabsPropsManager = function TabsPropsManager(ins, props) {
          var dir = props.dir,
            isVertical = props.isVertical;
          var result = {
            className: ins.getSetting('tablistClass') + ' ' + ins.getSetting(dir + "Class")
          };
          if (isVertical) {
            result.className += ' ' + ins.getSetting('verticalClass');
          }
          if (ins.getOption('accessibility')) {
            result.role = 'menu';
            result['aria-labelledby'] = buttonID;
            result.id = tablistID;
          }
          return result;
        };
        this._components.Tabs = /*#__PURE__*/React.forwardRef(this._components.TabsFactory.bind(undefined, function (ins) {
          return {
            Tab: _this._getTabComponent(),
            TabsPropsManager: function TabsPropsManager(props) {
              return _TabsPropsManager(ins, props);
            }
          };
        }));
        return components.Tabs;
      },
      _getTabComponent: function _getTabComponent() {
        var _this2 = this;
        return this._components.memomizeTab(this._components.TabFactory.bind(undefined, function (ins) {
          return {
            tabPropsManager: function tabPropsManager(props) {
              var originalProps = _this2._components.tabPropsManager(ins, props);
              originalProps['aria-controls'];
                originalProps['aria-expanded'];
                originalProps['aria-selected'];
                originalProps.id;
                var rest = _objectWithoutPropertiesLoose(originalProps, _excluded);
              if (rest.role) {
                rest.role = 'menuitem';
              }
              return rest;
            },
            tabInnerPropsManager: function tabInnerPropsManager(props) {
              return _extends$2({}, _this2._components.tabInnerPropsManager(ins, props), {
                popup: true
              });
            },
            closeIconPropsManager: function closeIconPropsManager() {
              return _this2._components.closeIconPropsManager(ins);
            }
          };
        }));
      },
      onButtonClick: function onButtonClick(ev, open) {
        ev.stopPropagation();
        this._registerBackdropEvent()._togglePopper(open);
      },
      _onBackdropClick: function _onBackdropClick() {
        this._closePopper();
      },
      _onSelectTab: function _onSelectTab() {
        this._closePopper();
      },
      _registerBackdropEvent: function _registerBackdropEvent() {
        window.document.removeEventListener('click', this._onBackdropClick, {
          once: true
        });
        window.document.addEventListener('click', this._onBackdropClick, {
          once: true
        });
        return this;
      },
      _CleanBackdropEvent: function _CleanBackdropEvent() {
        window.document.removeEventListener('click', this._onBackdropClick, {
          once: true
        });
        return this;
      },
      onDestroy: function onDestroy() {
        this._CleanBackdropEvent();
        this._cleanSelectEvent();
      },
      _registerSelectEvent: function _registerSelectEvent() {
        this._api.on('onSelect', this._onSelectTab);
      },
      _cleanSelectEvent: function _cleanSelectEvent() {
        (this == null ? void 0 : this._api) && this._api.off && this._api.off('onSelect', this._onSelectTab);
      },
      onMount: function onMount() {
        this._registerSelectEvent();
      }
    }))(this);
  }

  function _extends$1() { return _extends$1 = Object.assign ? Object.assign.bind() : function (n) { for (var e = 1; e < arguments.length; e++) { var t = arguments[e]; for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]); } return n; }, _extends$1.apply(null, arguments); }
  var moreButtonPlugin_buttonComponent = (function (deps) {
    if (deps === void 0) {
      deps = {};
    }
    var getDeps = function getDeps() {
      return _extends$1({
        Popper: Popper,
        Api: Api
      }, deps);
    };
    return /*#__PURE__*/React.memo(Button.bind(null, getDeps));
  });

  function MoreButtonPlugin_iconComponent(props) {
    var style = {};
    if (props.instance.getOption('direction') === 'rtl') {
      style.transform = 'rotate(180deg)';
    }
    return /*#__PURE__*/React.createElement("svg", {
      xmlns: "http://www.w3.org/2000/svg",
      height: "1em",
      viewBox: "0 0 512 512",
      style: style
    }, /*#__PURE__*/React.createElement("path", {
      fill: "gray",
      d: "M470.6 278.6c12.5-12.5 12.5-32.8 0-45.3l-160-160c-12.5-12.5-32.8-12.5-45.3 0s-12.5 32.8 0 45.3L402.7 256 265.4 393.4c-12.5 12.5-12.5 32.8 0 45.3s32.8 12.5 45.3 0l160-160zm-352 160l160-160c12.5-12.5 12.5-32.8 0-45.3l-160-160c-12.5-12.5-32.8-12.5-45.3 0s-12.5 32.8 0 45.3L210.7 256 73.4 393.4c-12.5 12.5-12.5 32.8 0 45.3s32.8 12.5 45.3 0z"
    }));
  }

  function _extends() { return _extends = Object.assign ? Object.assign.bind() : function (n) { for (var e = 1; e < arguments.length; e++) { var t = arguments[e]; for (var r in t) ({}).hasOwnProperty.call(t, r) && (n[r] = t[r]); } return n; }, _extends.apply(null, arguments); }
  function setComponents(deps, ctx, components) {
    var moreButtonPlugin_buttonComponent = deps.moreButtonPlugin_buttonComponent,
      ShowMoreTabs = deps.ShowMoreTabs,
      moreButtonPlugin_iconComponent = deps.moreButtonPlugin_iconComponent;
    setComponents.setSetting(ctx);
    setComponents.setDefaultOptions(ctx, components, moreButtonPlugin_buttonComponent, moreButtonPlugin_iconComponent);
    setComponents.setTablistView(components);
    setComponents.setMoreButtonPlugin(ctx, components, ShowMoreTabs, setComponents.ShowMoreButton, React.memo);
    setComponents.setTablistOverflow(components);
  }
  setComponents.ShowMoreButton = function ShowMoreButton(deps, props) {
    var ShowMoreTabs = deps.ShowMoreTabs,
      _deps$components = deps.components,
      useForceUpdate = _deps$components.useForceUpdate,
      useRootState = _deps$components.useRootState;
    useForceUpdate();
    var _useRootState = useRootState(),
      openTabIDs = _useRootState.openTabIDs,
      selectedTabID = _useRootState.selectedTabID;
    return /*#__PURE__*/React.createElement(ShowMoreTabs, _extends({}, props, {
      ctx: deps.ctx,
      openTabIDs: openTabIDs,
      selectedTabID: selectedTabID
    }), props.children /* eslint-disable-line react/prop-types */);
  };
  setComponents.setMoreButtonPlugin = function (ctx, components, ShowMoreTabs, ShowMoreButton, memo) {
    components.MoreButtonPlugin = memo(ShowMoreButton.bind(undefined, {
      ctx: ctx,
      components: components,
      ShowMoreTabs: ShowMoreTabs
    }), function () {
      return true;
    });
  };
  setComponents.setTablistOverflow = function (components) {
    if (!components.OriginalTablistOverflow) {
      components.OriginalTablistOverflow = components.TablistOverflow;
      components.TablistOverflow = function TablistOverflow(props) {
        return /*#__PURE__*/React.createElement(components.OriginalTablistOverflow, props, props.children /* eslint-disable-line react/prop-types */, /*#__PURE__*/React.createElement(components.MoreButtonPlugin, null));
      };
    }
  };
  setComponents.setTablistView = function (components) {
    components.TablistView = components.TablistViewFactory.bind(undefined, function (ins) {
      return {
        tablistViewPropsManager: function tablistViewPropsManager() {
          var attrs = components.tablistViewPropsManager(ins);
          attrs.className += ' rc-dyn-tabs-responsive';
          return attrs;
        }
      };
    });
  };
  setComponents.setDefaultOptions = function (ctx, components, moreButtonPlugin_buttonComponent, moreButtonPlugin_iconComponent) {
    ctx.optionsManager.options = Object.assign({
      moreButtonPlugin_buttonComponent: moreButtonPlugin_buttonComponent({
        ctx: ctx,
        components: components
      }),
      moreButtonPlugin_iconComponent: moreButtonPlugin_iconComponent,
      moreButtonPlugin_buttonTooltip: 'show more'
    }, ctx.optionsManager.options);
  };
  setComponents.setSetting = function (ctx) {
    ctx.optionsManager.setting.showMoreContainerClass = 'rc-dyn-tabs-showmorebutton-container';
    ctx.optionsManager.setting.showMoreButtonClass = 'rc-dyn-tabs-showmorebutton';
    ctx.optionsManager.setting.showMorePopperClass = 'rc-dyn-tabs-popper';
  };

  var index = setComponents.bind(undefined, {
    moreButtonPlugin_buttonComponent: moreButtonPlugin_buttonComponent,
    ShowMoreTabs: ShowMoreTabs,
    moreButtonPlugin_iconComponent: MoreButtonPlugin_iconComponent
  });

  return index;

}));
//# sourceMappingURL=more-button-plugin.umd.js.map
